﻿<%
'------------------------------------------------------------------------------------------------------------------------------------------
' SOOP 2.0 - Phoenix (TNG)
'
' Revision 2.00 - initial 15/02/2008
'
' SOOPPortal All Rights Reserved - Other copyrights as mentioned in sources or joined licences
'------------------------------------------------------------------------------------------------------------------------------------------

'******************************************************************************************************************************************
'* Original author: 		Nick DelMedico (www.pixel420.com)
'* Properties:
'*	.ElementID				- The name/id of the select element (default is "selCombo1")
'*	.SelectedValue			- The value of the option to show as selected (default is none)
'*	.DefaultText			- The text displayed for the default option (default is "- Please Select -")
'*	.ComboSize				- The size of the select Combo (default is "1")
'*	.ClassName				- Name of a CSS class to apply to the Combo (default is none)
'*	.Style					- CSS style to apply to the Combo (default is none)
'*	.DisplayDefault			- Boolean whether to display the default option (default is false)
'*	.MultipleSelect			- Boolean whether the Combo is multiple select (default is false)
'*	.Disabled 				- Boolean whether the Combo is disabled (default is false)
'* Methods:
'*	.BuildComboFromFile		- Builds Combo from a file. (Arguments: Path to file)
'*	.BuildComboFromDB		- Builds Combo from DB query. (Arguments: Connection Object, SQL Query)
'*	.AddOption				- Adds an option to Combo. (Arguments: Option Value, Option Display Text)
'*	.AddEvent				- Adds an event to the Combo. (Arguments: Event, Event Action)
'*	.CreateCombo			- Outputs Combo to browser (Arguments: None)
'*	.ReturnCombo			- Returns a string representation of the Combo (Arguments: None)
'*
'* Additonally extended for Phoenix:
'*	.BuildComboFromFolder		- Builds Combo from existing folder files
'*	.BuildComboFromDBIndent		- Builds Combo from existing DB with some indenting
'*	.Title property				- so we can use a generic jquery js function to do the needed jump
'*
'********************************************************************************************************************************************

class class_comboBuilder ':: class_string, class_baseUtils, class_filters, class_DAL, class_objectsPool

	private p_strElemID,   p_strSelected,  p_strDefaultText
	private p_strOutput,   p_strMultiple,  p_strDisabled, p_strIndented
	private p_strEvents,   p_strClassName, p_strStyle, p_strTitle
	private p_strReturn
	private p_intColBound, p_intComboSize
	private p_arrThisOpt
	private p_blnDispDef
	private p_objReader, p_objRS
	private p_folder, p_files, p_array, file

	private sub class_initialize()
		p_strElemID			= "selCombo1"
		p_strSelected		= ""
		p_strDefaultText	= " - Please Select - "
		p_strMultiple		= ""
		p_strDisabled		= ""
		p_intComboSize		= 1
		p_strTitle			= ""
		p_strEvents			= ""
		p_strClassName		= ""
		p_strStyle			= ""
		p_blnDispDef		= true
		'
		set str = new class_string
	end sub

	'* Properties -- all writable
	public property let ElementID(sElemID)
		p_strElemID = sElemID
	end property

	public property let SelectedValue(sSelVal)
		p_strSelected = sSelVal
	end property

	public property let DefaultText(sDefTxt)
		p_strDefaultText = sDefTxt
	end property

	public property let MultipleSelect(bMultiple)
		if bMultiple = true Then
			p_strMultiple = " multiple=""multiple"""
		else
			p_strMultiple = ""
		end if
	end property

	public property let ComboSize(iComboSize)
		if isNumeric(iComboSize) Then p_intComboSize = iComboSize
	end property

	public property let ClassName(sClass)
		p_strClassName = " class=""" & sclass & """"
	end property
	
	public property let Title(sTitle)
		p_strTitle = " title=""" & sTitle & """"
	end property

	public property let Style(sStyle)
		p_strStyle = " style=""" & sStyle & """"
	end property

	public property let DisplayDefault(bDisplay)
		if bDisplay = true OR bDisplay = false Then
			p_blnDispDef = bDisplay
		end if
	end property

	public property let Disabled(bDisable)
		if bDisable = true Then
			p_strDisabled = " disabled=""disabled"""
		else
			p_strDisabled = ""
		end if
	end property

	public sub AddOption(sValue, sDisplay)
		call addSelectOption(sValue, sDisplay, p_strSelected, "")
	end sub

	public sub AddEvent(sEvent, sAction)
		p_strEvents = p_strEvents & " " & sEvent & "=""" & sAction & """"
	end sub

	public sub BuildComboFromFile(sFilePath)
		call cor.obj.init("fso")
		set p_objReader = cor.obj.fso.openTextFile(sFilePath, 1, false)
		Do Until p_objReader.atEndOfStream
			p_arrThisOpt = split(p_objReader.ReadLine, ",")
			strXMLlabel = cor.lng.getKey("cor_xmlCombos",p_arrThisOpt(ubound(p_arrThisOpt)))
			if isNull(strXMLlabel) OR strXMLlabel = "" Then strXMLlabel = p_arrThisOpt(ubound(p_arrThisOpt))
			call addSelectOption(p_arrThisOpt(0), strXMLlabel, p_strSelected, "")
		Loop
		erase p_arrThisOpt
		p_objReader.close
		set p_objReader = nothing
	end sub

	public sub BuildComboFromFolder(sFolder,sExtension)
		call cor.obj.init("fso")
		set p_folder = cor.obj.fso.GetFolder(server.mappath(sFolder))
		set p_files  = p_folder.Files
		for each file in p_files
			if instr(file.name, sExtension) > 0 Then
				p_arrThisOpt = split(lcase(replace(file.name, sExtension, "")),",")
				call addSelectOption(p_arrThisOpt(0), cor.butil.properCase(p_arrThisOpt(ubound(p_arrThisOpt))), p_strSelected, "")
			end if
		next
		set p_files  = nothing
		set p_folder = nothing
	end sub

	public sub BuildComboFromDB(sQuery)
		if isEmpty(sQuery) Then exit Sub
		cor.dal.open p_array, sQuery, array()
		if NOT cor.butil.eof(p_array) Then
				for p_intColBound = 0 To ubound(p_array,2)
					'call addSelectOption(trim(p_array(0,p_intColBound)), cor.butil.htmlEncode(trim(p_array(1,p_intColBound))), p_strSelected, "")
					call addSelectOption(trim(p_array(0,p_intColBound)), trim(p_array(1,p_intColBound)), p_strSelected, "")
				next
		end if
	end sub

	public sub BuildComboFromDBIndent(sQuery)
		if isEmpty(sQuery) Then exit Sub
		cor.dal.open p_array, sQuery, array()
		if NOT cor.butil.eof(p_array) Then
			for p_intColBound = 0 To ubound(p_array,2)
				'call addSelectOption(trim(p_array(0,p_intColBound)), cor.butil.htmlEncode(trim(p_array(1,p_intColBound))), p_strSelected, p_array(4,p_intColBound))
				call addSelectOption(trim(p_array(0,p_intColBound)), trim(p_array(1,p_intColBound)), p_strSelected, p_array(4,p_intColBound))
			next
		end if
	end sub

	public sub CreateCombo()
		str.add "<select name=""" & p_strElemID & """ id=""" & p_strElemID & """ size=""" & p_intComboSize & """" & p_strEvents & p_strTitle & p_strClassName & p_strStyle & p_strMultiple & p_strDisabled & ">" 
		if p_blnDispDef then str.add "<option value="""">" & p_strDefaultText & "</option>"
		str.add p_strOutput & "</select>"
		response.write str.build
	end sub

	public function ReturnCombo()
		str.add "<select name=""" & p_strElemID & """ id=""" & p_strElemID & """ size=""" & p_intComboSize & """" & p_strEvents & p_strTitle & p_strClassName & p_strStyle & p_strMultiple & p_strDisabled & ">"
		if p_blnDispDef Then str.add "<option value="""">" & p_strDefaultText & "</option>"
		str.add p_strOutput & "</select>"
		ReturnCombo = str.build
	end function


	Private Sub addSelectOption(sValue, sDisplay, sSelected, sClass)
		dim strRepeat, x_i,sItem
		if isEmpty(sSelected) or isnull (sSelected) then sSelected = " "
		sItem = split(sSelected,",")
		if sclass = "" Then
			str.adn "<option value=""" & sValue & """"
			For x_i =0 to Ubound(sItem)
				if trim(lcase(sValue))=trim(lcase(sItem(x_i))) Then str.adn " selected=""selected"""
			Next
			str.add ">" & sDisplay & "</option>"
		Else
			str.adn "<option style=""padding-left:" & (sClass*10)+5 & "px;"" value=""" & sValue & """"
			For x_i =0 to Ubound(sItem)
				if trim(lcase(sValue))=trim(lcase(sItem(x_i))) Then str.adn " selected=""selected"""
			Next
			str.add ">" & sDisplay & "</option>"
		end If
		p_strOutput = p_strOutput & str.build
	End Sub


	private sub class_terminate()
		p_strOutput = empty
		p_strReturn = empty
		set str	= nothing
	end sub
	'
	private str
	private strXMLlabel
	'
end class

'<#late-binding hook
function boot_class_comboBuilder : boot_class_comboBuilder = true : end function
'/>

%>