<%
'------------------------------------------------------------------------------------------------------------------------------------------
' SOOP 2.0 - Phoenix (TNG)
'
'Revision 2.0001 - initial 26/07/2008
'
' SOOPPortal All Rights Reserved - Other copyrights as mentioned in sources or joined licences
'------------------------------------------------------------------------------------------------------------------------------------------
%>
<%
class class_security ':: class_string, class_filters, class_io, class_audit
	'    
    '------------------------------------------------------------------------------------------------------------------------------------------
    '
    '
    '
    '------------------------------------------------------------------------------------------------------------------------------------------
    '--
    '-- function: validateUserAccess(*)
    '-- Description: This will check if a user is accessing your website from a
    '--              browser or not, verify they are using post (for forms to prevent
    '--              people using GET), and prevents header injection.
    '-- Returns: Returns true if no security breach has been detected
    '--          Returns false if a security breach has been detected
    '-- Example: if validateUserAccess(true, true, true, true) Then
    '--            '--Security Ok
    '--          end if
    '--
    '--
    '-- function: validateUserAccessAll(*)
    '-- Description: The same as validateUserAccess(*) with all options enabled.
    '-- Returns: Returns true if no security breach has been detected
    '--          Returns false if a security breach has been detected
    '-- Example: if validateUserAccessAll(true) Then
    '--            '--Security Ok
    '--          end if
    '--
    '--
    '-- function: authorizedDomain(*)
    '-- Description: This will allow you to block certain domain from viewing your
    '--              web pages. Simply provide an array of domains you want blocked
    '--              and it will check if the person is coming from that site.
    '-- Returns: Returns true if the domain is not blocked
    '--          Returns false if the domain is blocked
    '-- Example: arrDomains = array("www.google.com","www.yahoo.com")
    '--          if Not ValidateDomain(true, arrDomains, false) Then
    '--            '--Blocked Domain, do as you will
    '--          end if
    '--
    '--
    '-- function: SQLInjectionBlock(*)
    '-- Description: This will prevent a security breach known as SQL Injection.
    '--              The function will strip common SQL specific characters from the
    '--              string passed in.
    '-- Returns: Returns the buffer without the SQL specific characters in it
    '-- Example: strPassword = SQLInjectionBlock(true, strPassword, 0)
    '--
    '--
    '-- function: ValidateSessionID(*)
    '-- Description: Each form will save the Session ID in a hidden input
    '--              if the value is different from the value we receive when the form is posted
    '--              we have a security breach.
    '-- Returns: Returns true if the values are the same
    '--

    '-------------------------------------------------------------------------------
    '--
    '-- function: validateUserAccessAll(*)
    '-- Version: 1.0
    '--
    '-------------------------------------------------------------------------------
    public function validateUserAccessAll(ByVal blnLogBadrequest, ByVal lngSessionID)
      	validateUserAccessAll = validateUserAccess(blnLogBadrequest, true, true, true, true, lngSessionID)
    end function
    
    '-------------------------------------------------------------------------------
    '--
    '-- function: ValidateSessionID(*)
    '-- Version: 1.0
    '--
    '-------------------------------------------------------------------------------
    public function validateSessionID(ByVal lngSessionID, ByVal blnLogBadrequest)
    	validateSessionID =  true
    	if lngSessionID = session.sessionID Then
    		validateSessionID = true
    	else
          if blnLogBadrequest Then
            call appendSecurityLog("Session ID Problem - " & lngSessionID & "-" & session.sessionID)	
          end if
    	end if
    end function
                                       
    '-------------------------------------------------------------------------------
    '--
    '-- function: validateUserAccess(*)
    '-- Version: 1.0
    '--
    '-------------------------------------------------------------------------------
    public function validateUserAccess(ByVal blnLogBadrequest, _
                                       ByVal blnCheckForAgent, _
                                       ByVal blnCheckMethod, _
                                       ByVal blnCheckHeaderInjection, _
                                       ByVal blncheckSessionionID, _
                                       ByVal lngSessionID)
    
      '--Make sure valid boolean data types are being passed in
      if NOT validBooleanParameter(blnLogBadrequest) OR _
         NOT validBooleanParameter(blnCheckForAgent) OR _
         NOT validBooleanParameter(blnCheckMethod) OR _
         NOT validBooleanParameter(blncheckSessionionID) OR _
         NOT validBooleanParameter(blnCheckHeaderInjection) Then
        exit function
      end if
      
      '--Make sure if Session ID has to be checked a valid one has been passed
      '--if asked for, check it
      if blncheckSessionionID AND NOT cor.filter.checkLng(lngSessionID) > 0 Then
        exit function
      else
      	if blncheckSessionionID AND NOT ValidateSessionID(lngSessionID,blnLogBadrequest) Then
      	  exit function
      	end if
      end if
    
      '--Make sure someone is using a web browser to access the page.
      if blnCheckForAgent AND CheckForNoAgent(blnLogBadrequest) Then
        exit function
      end if
    
      '--Make sure someone is using the method we want
      if blnCheckMethod AND CheckForInvalidMethod(blnLogBadrequest) Then
        exit function
      end if
    
      '--Make sure someone is not trying to inject the header contents.
      if blnCheckHeaderInjection AND CheckForHeaderInjection(blnLogBadrequest) Then
        exit function
      end if
    
      '--There was no violation encountered
      validateUserAccess = true
    end function
    '-------------------------------------------------------------------------------
    '--
    '-- function: SQLInjectionBlock(*)
    '-- Version: 1.0
    '--
    '-- enmStrength Values
    '--     Low(0) - Common SQL characters
    '--     High(1) - Common SQL characters, with quotes, and keywords
    '--
    '-------------------------------------------------------------------------------
    private function SQLInjectionBlock(ByVal blnLogBadrequest, ByVal strBuffer, ByVal enmStrength)
      dim blnAlreadyLogged
      dim arrExtended
      dim arrCommon
      dim lngID
    
      '--Make sure we have a valid buffer before working with it
      if trim(strBuffer) = "" Then
        exit function
      end if
    
      '--Setup the common array values (you can modify as needed)
      arrCommon = array("'", """", ";", "*", ",", "--", "(", ")", "=")
    
      '--Setup the extended array values (you can modify as needed, I just setup a few)
      '--
      '--Please note that the high strength could have undesired effects.
      '--if someone has a dog named "Thor" and they use that as a username then
      '--it will be striped to "Th" so be careful. So we should only use it for fields that needs this high security....
      '--
      '--
      arrExtended = array("SELECT", "FROM", "WHERE", "AND", "OR", "JOIN", "UNION", "DELETE", "UPDATE", "LIKE", "DROP", "CREATE", "MODIFY", "ALTER", "CAST", "XP_", "--")
    
      '--Clear any and all instances of the of the buffer that match the
      '--the common array
      for lngID = lBound(arrCommon) To ubound(arrCommon)
        '--if we are logging this and it has not already been logged and the
        '--buffer contains a match then log it
        if blnLogBadrequest AND NOT blnAlreadyLogged AND instr(1, strBuffer, arrCommon(lngID)) > 0 Then
          call appendSecurityLog("SQL Injection - (" & strBuffer & ")")
          blnAlreadyLogged = true
        end if
        strBuffer = replace(strBuffer, arrCommon(lngID), "")
      Next
    
      '--if the extra strength is requested, then setup that array too
      if enmStrength > 0 Then
        '--Clear any and all instances of the of the buffer that match the
        '--the extended array
        for lngID = lBound(arrExtended) To ubound(arrExtended)
          '--if we are logging this and it has not already been logged and the
          '--buffer contains a match then log it
          if blnLogBadrequest AND NOT blnAlreadyLogged AND instr(1, strBuffer, arrCommon(lngID)) > 0 Then
            call appendSecurityLog("SQL Injection - (" & strBuffer & ")")
            blnAlreadyLogged = true
          end if
          strBuffer = replace(strBuffer, arrExtended(lngID), "")
        Next
      end if
    
      '--Return the modified buffer
      SQLInjectionBlock = strBuffer
    
      '--Clear the resources used by the arrays
      erase arrCommon
      erase arrExtended
    
    end function
    
    '-------------------------------------------------------------------------------
    '--
    '-- function: CheckForNoAgent(*)
    '-- Version: 1.0
    '--
    '-------------------------------------------------------------------------------
    private function CheckForNoAgent(ByVal blnLogBadrequest)
    
      '--Make sure the request is coming from a web browser and
      '--not anything else.
      if IsEmpty(request.serverVariables("HTTP_USER_AGENT")) OR _
         isNull(request.serverVariables("HTTP_USER_AGENT")) Then
        if blnLogBadrequest Then
          call appendSecurityLog("No User Agent")
        end if
        CheckForNoAgent = true
      end if
    
    end function
    
    '-------------------------------------------------------------------------------
    '--
    '-- function: CheckForInvalidMethod(*)
    '-- Version: 1.0
    '--
    '-------------------------------------------------------------------------------
    private function CheckForInvalidMethod(ByVal blnLogBadrequest)
    
      '--Make sure the information we are receiving has been
      '--posted from a form. This will keep people from using GET.
      if NOT cor_bPostback Then
        if blnLogBadrequest Then
          	call appendSecurityLog("Invalid request Method")
        end if
        CheckForInvalidMethod = true
      end if
    
    end function
    
    '-------------------------------------------------------------------------------
    '--
    '-- function: CheckForHeaderInjection(*)
    '-- Version: 1.0
    '--
    '-------------------------------------------------------------------------------
    private function CheckForHeaderInjection(ByVal blnLogBadrequest)
      dim arrInvalidHeaderEntries
      dim strQueryBuffer
      dim strFormBuffer
      dim lngID
    
      '--Get the requests from the form and URL
      strFormBuffer = cor.io.fa
      strQueryBuffer = cor.io.qa
    
      '--Make sure there is something in at least one of the
      '--buffers. if the buffers are empty then no attack is
      '--being carried out.
      if strFormBuffer = "" AND strQueryBuffer = "" Then
        exit function
      end if
    
      '--Build an array of possible header injections
      arrInvalidHeaderEntries = array("Content-Type:", _
                                      "MIME-Version:", _
                                      "Content-Transfer-Encoding:", _
                                      "bcc:", _
                                      "cc:")
    
      '--Loop through each item in the array
      for lngID = 0 To ubound(arrInvalidHeaderEntries)
        '--Check and see if the URL data has a match, if so then
        '--we set the flag to indicate an injection attempt is taking
        '--place. After that we exit the for loop since we do not need
        '--to continue checking.
        if strQueryBuffer <> "" Then
          if instr(1, strQueryBuffer, arrInvalidHeaderEntries(lngID), 1) > 0 Then
            if blnLogBadrequest Then
              	call appendSecurityLog("Possible Header Injection Attempt (QueryString)")
            end if
            CheckForHeaderInjection = true
            exit For
          end if
        end if
    
        '--Check and see if the form data has a match, if so then
        '--we set the flag to indicate an injection attempt is taking
        '--place. After that we exit the for loop since we do not need
        '--to continue checking.
        if strFormBuffer <> "" Then
          if instr(1, strFormBuffer, arrInvalidHeaderEntries(lngID), 1) > 0 Then
            if blnLogBadrequest Then
              	call appendSecurityLog("Possible Header Injection Attempt (Form)")
            end if
            CheckForHeaderInjection = true
            exit For
          end if
        end if
      Next
    
      '--Clear the array data from memory
      call Erase(arrInvalidHeaderEntries)
    
      '--Clear the local variables
      strQueryBuffer = ""
      strFormBuffer = ""
    
    end function
    
    '-------------------------------------------------------------------------------
    '--
    '-- function: appendSecurityLog(*)
    '-- Version: 1.0
    '--
    '-------------------------------------------------------------------------------
    private sub appendSecurityLog(ByVal strViolation)
      dim objFileSystem
      dim objFileHandle
      dim intFileFlag
      dim strLogFile
      dim strPath
      '
      '--Write the information to the security log
      str.clear
      str.add vbCrlf
      str.add "[SECURITY ALERT: " 	 & Month(Date) & "." & Day(Date) & "." & Year(Date) & " - " & Time() & "]"
      str.add "    Violation: " 	 & strViolation
      str.add "    Port: " 			 & request.serverVariables("SERVER_PORT")
      str.add "    Remote IP: " 	 & request.serverVariables("REMOTE_ADDR")
      str.add "    Remote Host: " 	 & request.serverVariables("REMOTE_HOST")
      str.add "    request Method: " & request.serverVariables("REQUEST_METHOD")
      str.add "    Query: " 		 & request.serverVariables("QUERY_STRING")
      str.add "    HTTP Referer: "   & request.serverVariables("HTTP_REFERER")
      str.add "    Agent: " 		 & request.serverVariables("HTTP_USER_AGENT")
	  str.add "************************************************************"
      str.add vbCrlf
      '
      call cor.audit.writeLogFile(str.build, "security.txt")
      '
    end sub
    '
    '-------------------------------------------------------------------------------
    '--
    '-- function: DomainFromURL(*)
    '-- Version: 1.0
    '--
    '-------------------------------------------------------------------------------
    private function DomainFromURL(strURL)
      dim arrTokens
    
      '--if the URL contains http:// then parse out the domain
      '--name by splitting the URL.
      if instr(strURL, "//") > 0 Then
        arrTokens = split(strURL, "/")
        DomainFromURL = arrTokens(2)
        erase arrTokens
      end if
    
    end function
    
    '-------------------------------------------------------------------------------
    '--
    '-- function: validBooleanParameter(*)
    '-- Version: 1.0
    '--
    '-------------------------------------------------------------------------------
    private function validBooleanParameter(blnParameter)
    
      '--Make sure the parameter passed in is valid
      if IsEmpty(blnParameter) Or isNull(blnParameter) Then
        exit function
      end if
    
      '--Make sure the parameter passed in is a valid boolean
      '--data type
      if blnParameter <> true AND _
         blnParameter <> false AND _
         blnParameter <> 1 AND _
         blnParameter <> -1 AND _
         blnParameter <> 0 Then
        exit function
      end if
    
      '--set the return to indicate it is a valid boolean data type
      validBooleanParameter = true
    
    end function
    
    '-------------------------------------------------------------------------------
    '--
    '-- function: ValidArrayParameter(*)
    '-- Version: 1.0
    '--
    '-------------------------------------------------------------------------------
    private function ValidArrayParameter(arrParameter)
      '--Make sure the parameter passed in is valid
      if IsEmpty(arrParameter) Or isNull(arrParameter) Then
        exit function
      end if
      '--Return the IsArray result
      validBooleanParameter = Isarray(arrParameter)
    end function

    '-------------------------------------------------------------------------------
    '--
    '-- function: inarray(8) - check if a string/value is in an array
    '-- Version: 1.0
    '--
    '-------------------------------------------------------------------------------
    private function inarray(element, arr)
        For i=0 To ubound(arr)
            if trim(arr(i)) = trim(element) Then
                inArray = True
                exit function
            else
                inArray = False
            end if 
        next
    end function 
	'
	private sub class_initialize
		set str = new class_string
	end sub
	'
	private sub class_terminate
		set str = nothing
	end sub
	'
	private str
	'
end class

'<#late-binding hook
function boot_class_security : boot_class_security = true : end function
'/>
%>