<%
'------------------------------------------------------------------------------------------------------------------------------------------
' SOOP 2.0 - Phoenix (TNG)
'
'Revision 2.0001 - initial 26/02/2008
'
' SOOPPortal All Rights Reserved - Other copyrights as mentioned in sources or joined licences
'------------------------------------------------------------------------------------------------------------------------------------------
%>
<%
' See the VB6 project that accompanies this sample for full code comments on how
' it works.
'
' ASP VBScript code for generating a SHA256 'digest' or 'signature' of a string. The
' MD5 algorithm is one of the industry standard methods for generating digital
' signatures. It is genericall y known as a digest, digital signature, one-way
' encryption, hash or checksum algorithm. A common use for SHA256 is for password
' encryption as it is one-way in nature, that does not mean that your passwords
' are not free from a dictionary attack. 
'
' if you are using the routine for passwords, you can make it a little more secure
' by concatenating some known random characters to the password before you generate
' the signature and on subsequent tests, so even if a hacker knows you are using
' SHA-256 for your passwords, the random characters will make it harder to dictionary
' attack.
'
' NOTE: Due to the way in which the string is processed the routine assumes a
' single byte character set. VB passes unicode (2-byte) character strings, the
' ConvertToWordArray function uses on the first byte for each character. This
' has been done this way for ease of use, to make the routine truely portable
' you could accept a byte array instead, it would then be up to the call ing
' routine to make sure that the byte array is generated from their string in
' a manner consistent with the string type.
'
' This is 'free' software with the following restrictions:
'
' You may not redistribute this code as a 'sample' or 'demo'. However, you are free
' to use the source code in your own code, but you may not claim that you created
' the sample code. It is expressly forbidden to sell or profit from this source code
' other than by the knowledge gained or the enhanced value added by your own code.
'
' Use of this software is also done so at your own risk. The code is supplied as
' is without warranty or guarantee of any kind.
'
' Should you wish to commission some derivative work based on this code provided
' here, or any consultancy work, please do not hesitate to contact us.
'
' Web Site:  http://www.frez.co.uk
' E-mail:	sales@frez.co.uk

class class_sha256

   public function enc256(sMessage)
        dim HASH(7)
        dim M
        dim W(63)
        dim a
        dim b
        dim c
        dim d
        dim e
        dim f
        dim g
        dim h
        dim i
        dim j
        dim T1
        dim T2
        
        HASH(0) = &H6A09E667
        HASH(1) = &HBB67AE85
        HASH(2) = &H3C6EF372
        HASH(3) = &HA54FF53A
        HASH(4) = &H510E527F
        HASH(5) = &H9B05688C
        HASH(6) = &H1F83D9AB
        HASH(7) = &H5BE0CD19
        
        M = ConvertToWordarray(sMessage)
        
        for i = 0 To ubound(M) Step 16
            a = HASH(0)
            b = HASH(1)
            c = HASH(2)
            d = HASH(3)
            e = HASH(4)
            f = HASH(5)
            g = HASH(6)
            h = HASH(7)
            
            for j = 0 To 63
                if j < 16 Then
                    W(j) = M(j + i)
                else
                    W(j) = AddUnsigned(AddUnsigned(AddUnsigned(Gamma1(W(j - 2)), W(j - 7)), Gamma0(W(j - 15))), W(j - 16))
                end if
                    
                T1 = AddUnsigned(AddUnsigned(AddUnsigned(AddUnsigned(h, Sigma1(e)), Ch(e, f, g)), K(j)), W(j))
                T2 = AddUnsigned(Sigma0(a), Maj(a, b, c))
                
                h = g
                g = f
                f = e
                e = AddUnsigned(d, T1)
                d = c
                c = b
                b = a
                a = AddUnsigned(T1, T2)
            Next
            
            HASH(0) = AddUnsigned(a, HASH(0))
            HASH(1) = AddUnsigned(b, HASH(1))
            HASH(2) = AddUnsigned(c, HASH(2))
            HASH(3) = AddUnsigned(d, HASH(3))
            HASH(4) = AddUnsigned(e, HASH(4))
            HASH(5) = AddUnsigned(f, HASH(5))
            HASH(6) = AddUnsigned(g, HASH(6))
            HASH(7) = AddUnsigned(h, HASH(7))
        Next
        
        enc256 = LCase(Right("00000000" & Hex(HASH(0)), 8) & Right("00000000" & Hex(HASH(1)), 8) & Right("00000000" & Hex(HASH(2)), 8) & Right("00000000" & Hex(HASH(3)), 8) & Right("00000000" & Hex(HASH(4)), 8) & Right("00000000" & Hex(HASH(5)), 8) & Right("00000000" & Hex(HASH(6)), 8) & Right("00000000" & Hex(HASH(7)), 8))
    end function
    
    private function LShift(lValue, iShiftBits)
        if iShiftBits = 0 Then
            LShift = lValue
            exit function
        elseif iShiftBits = 31 Then
            if lValue And 1 Then
                LShift = &H80000000
            else
                LShift = 0
            end if
            exit function
        elseif iShiftBits < 0 Or iShiftBits > 31 Then
            err.raise 6
        end if
        
        if (lValue And m_l2Power(31 - iShiftBits)) Then
            LShift = ((lValue And m_lOnBits(31 - (iShiftBits + 1))) * m_l2Power(iShiftBits)) Or &H80000000
        else
            LShift = ((lValue And m_lOnBits(31 - iShiftBits)) * m_l2Power(iShiftBits))
        end if
    end function
    
    private function RShift(lValue, iShiftBits)
        if iShiftBits = 0 Then
            RShift = lValue
            exit function
        elseif iShiftBits = 31 Then
            if lValue And &H80000000 Then
                RShift = 1
            else
                RShift = 0
            end if
            exit function
        elseif iShiftBits < 0 Or iShiftBits > 31 Then
            err.raise 6
        end if
        
        RShift = (lValue And &H7FFFFFFE) \ m_l2Power(iShiftBits)
        
        if (lValue And &H80000000) Then
            RShift = (RShift Or (&H40000000 \ m_l2Power(iShiftBits - 1)))
        end if
    end function
    
    private function AddUnsigned(lX, lY)
        dim lX4
        dim lY4
        dim lX8
        dim lY8
        dim lResult
     
        lX8 = lX And &H80000000
        lY8 = lY And &H80000000
        lX4 = lX And &H40000000
        lY4 = lY And &H40000000
     
        lResult = (lX And &H3FFFFFFF) + (lY And &H3FFFFFFF)
     
        if lX4 And lY4 Then
            lResult = lResult Xor &H80000000 Xor lX8 Xor lY8
        elseif lX4 Or lY4 Then
            if lResult And &H40000000 Then
                lResult = lResult Xor &HC0000000 Xor lX8 Xor lY8
            else
                lResult = lResult Xor &H40000000 Xor lX8 Xor lY8
            end if
        else
            lResult = lResult Xor lX8 Xor lY8
        end if
     
        AddUnsigned = lResult
    end function
    
    private function Ch(x, y, z)
        Ch = ((x And y) Xor ((Not x) And z))
    end function
    
    private function Maj(x, y, z)
        Maj = ((x And y) Xor (x And z) Xor (y And z))
    end function
    
    private function S(x, n)
        S = (RShift(x, (n And m_lOnBits(4))) Or LShift(x, (32 - (n And m_lOnBits(4)))))
    end function
    
    private function R(x, n)
        R = RShift(x, clng(n And m_lOnBits(4)))
    end function
    
    private function Sigma0(x)
        Sigma0 = (S(x, 2) Xor S(x, 13) Xor S(x, 22))
    end function
    
    private function Sigma1(x)
        Sigma1 = (S(x, 6) Xor S(x, 11) Xor S(x, 25))
    end function
    
    private function Gamma0(x)
        Gamma0 = (S(x, 7) Xor S(x, 18) Xor R(x, 3))
    end function
    
    private function Gamma1(x)
        Gamma1 = (S(x, 17) Xor S(x, 19) Xor R(x, 10))
    end function
    
    private function ConvertToWordarray(sMessage)
        dim lMessageLength
        dim lNumberOfWords
        dim lWordarray()
        dim lBytePosition
        dim lByteCount
        dim lWordCount
        dim lByte
        
        const MODULUS_BITS = 512
        const CONGRUENT_BITS = 448
        
        lMessageLength = len(sMessage)
        
        lNumberOfWords = (((lMessageLength + ((MODULUS_BITS - CONGRUENT_BITS) \ BITS_TO_A_BYTE)) \ (MODULUS_BITS \ BITS_TO_A_BYTE)) + 1) * (MODULUS_BITS \ BITS_TO_A_WORD)
        redim lWordarray(lNumberOfWords - 1)
        
        lBytePosition = 0
        lByteCount = 0
        Do Until lByteCount >= lMessageLength
            lWordCount = lByteCount \ BYTES_TO_A_WORD
            
            lBytePosition = (3 - (lByteCount Mod BYTES_TO_A_WORD)) * BITS_TO_A_BYTE
            
            lByte = AscB(mid(sMessage, lByteCount + 1, 1))
            
            lWordarray(lWordCount) = lWordarray(lWordCount) Or LShift(lByte, lBytePosition)
            lByteCount = lByteCount + 1
        Loop
    
        lWordCount = lByteCount \ BYTES_TO_A_WORD
        lBytePosition = (3 - (lByteCount Mod BYTES_TO_A_WORD)) * BITS_TO_A_BYTE
    
        lWordarray(lWordCount) = lWordarray(lWordCount) Or LShift(&H80, lBytePosition)
    
        lWordarray(lNumberOfWords - 1) = LShift(lMessageLength, 3)
        lWordarray(lNumberOfWords - 2) = RShift(lMessageLength, 29)
        
        ConvertToWordArray = lWordArray
    end function

	private sub class_initialize()
 		'--- arrays
     	redim m_lOnBits(30)
    	redim m_l2Power(30)
    	redim K(63)
    	'--- constants
    	BITS_TO_A_BYTE  = 8
    	BYTES_TO_A_WORD = 4
    	BITS_TO_A_WORD  = 32
		'
        m_lOnBits(0)  = clng(1)
        m_lOnBits(1)  = clng(3)
        m_lOnBits(2)  = clng(7)
        m_lOnBits(3)  = clng(15)
        m_lOnBits(4)  = clng(31)
        m_lOnBits(5)  = clng(63)
        m_lOnBits(6)  = clng(127)
        m_lOnBits(7)  = clng(255)
        m_lOnBits(8)  = clng(511)
        m_lOnBits(9)  = clng(1023)
        m_lOnBits(10) = clng(2047)
        m_lOnBits(11) = clng(4095)
        m_lOnBits(12) = clng(8191)
        m_lOnBits(13) = clng(16383)
        m_lOnBits(14) = clng(32767)
        m_lOnBits(15) = clng(65535)
        m_lOnBits(16) = clng(131071)
        m_lOnBits(17) = clng(262143)
        m_lOnBits(18) = clng(524287)
        m_lOnBits(19) = clng(1048575)
        m_lOnBits(20) = clng(2097151)
        m_lOnBits(21) = clng(4194303)
        m_lOnBits(22) = clng(8388607)
        m_lOnBits(23) = clng(16777215)
        m_lOnBits(24) = clng(33554431)
        m_lOnBits(25) = clng(67108863)
        m_lOnBits(26) = clng(134217727)
        m_lOnBits(27) = clng(268435455)
        m_lOnBits(28) = clng(536870911)
        m_lOnBits(29) = clng(1073741823)
        m_lOnBits(30) = clng(2147483647)
        '
        m_l2Power(0)  = clng(1)
        m_l2Power(1)  = clng(2)
        m_l2Power(2)  = clng(4)
        m_l2Power(3)  = clng(8)
        m_l2Power(4)  = clng(16)
        m_l2Power(5)  = clng(32)
        m_l2Power(6)  = clng(64)
        m_l2Power(7)  = clng(128)
        m_l2Power(8)  = clng(256)
        m_l2Power(9)  = clng(512)
        m_l2Power(10) = clng(1024)
        m_l2Power(11) = clng(2048)
        m_l2Power(12) = clng(4096)
        m_l2Power(13) = clng(8192)
        m_l2Power(14) = clng(16384)
        m_l2Power(15) = clng(32768)
        m_l2Power(16) = clng(65536)
        m_l2Power(17) = clng(131072)
        m_l2Power(18) = clng(262144)
        m_l2Power(19) = clng(524288)
        m_l2Power(20) = clng(1048576)
        m_l2Power(21) = clng(2097152)
        m_l2Power(22) = clng(4194304)
        m_l2Power(23) = clng(8388608)
        m_l2Power(24) = clng(16777216)
        m_l2Power(25) = clng(33554432)
        m_l2Power(26) = clng(67108864)
        m_l2Power(27) = clng(134217728)
        m_l2Power(28) = clng(268435456)
        m_l2Power(29) = clng(536870912)
        m_l2Power(30) = clng(1073741824)
            
        K(0)  = &H428A2F98
        K(1)  = &H71374491
        K(2)  = &HB5C0FBCF
        K(3)  = &HE9B5DBA5
        K(4)  = &H3956C25B
        K(5)  = &H59F111F1
        K(6)  = &H923F82A4
        K(7)  = &HAB1C5ED5
        K(8)  = &HD807AA98
        K(9)  = &H12835B01
        K(10) = &H243185BE
        K(11) = &H550C7DC3
        K(12) = &H72BE5D74
        K(13) = &H80DEB1FE
        K(14) = &H9BDC06A7
        K(15) = &HC19BF174
        K(16) = &HE49B69C1
        K(17) = &HEFBE4786
        K(18) = &HFC19DC6
        K(19) = &H240CA1CC
        K(20) = &H2DE92C6F
        K(21) = &H4A7484AA
        K(22) = &H5CB0A9DC
        K(23) = &H76F988DA
        K(24) = &H983E5152
        K(25) = &HA831C66D
        K(26) = &HB00327C8
        K(27) = &HBF597FC7
        K(28) = &HC6E00BF3
        K(29) = &HD5A79147
        K(30) = &H6CA6351
        K(31) = &H14292967
        K(32) = &H27B70A85
        K(33) = &H2E1B2138
        K(34) = &H4D2C6DFC
        K(35) = &H53380D13
        K(36) = &H650A7354
        K(37) = &H766A0ABB
        K(38) = &H81C2C92E
        K(39) = &H92722C85
        K(40) = &HA2BFE8A1
        K(41) = &HA81A664B
        K(42) = &HC24B8B70
        K(43) = &HC76C51A3
        K(44) = &HD192E819
        K(45) = &HD6990624
        K(46) = &HF40E3585
        K(47) = &H106AA070
        K(48) = &H19A4C116
        K(49) = &H1E376C08
        K(50) = &H2748774C
        K(51) = &H34B0BCB5
        K(52) = &H391C0CB3
        K(53) = &H4ED8AA4A
        K(54) = &H5B9CCA4F
        K(55) = &H682E6FF3
        K(56) = &H748F82EE
        K(57) = &H78A5636F
        K(58) = &H84C87814
        K(59) = &H8CC70208
        K(60) = &H90BEFFFA
        K(61) = &HA4506CEB
        K(62) = &HBEF9A3F7
        K(63) = &HC67178F2
	end sub

    private m_lOnBits
    private m_l2Power
    private K
	
    private BITS_TO_A_BYTE
    private BYTES_TO_A_WORD
    private BITS_TO_A_WORD
 
end class

'<#late-binding hook
function boot_class_sha256 : boot_class_sha256 = true : end function
'/>
%>
