(function() {
	// Load plugin specific language pack
	//tinymce.PluginManager.requireLangPack('example');

	tinymce.create('tinymce.plugins.MaxChars', {
		/**
		 * Initializes the plugin, this will be executed after the plugin has been created.
		 * This call is done before the editor instance has finished it's initialization so use the onInit event
		 * of the editor instance to intercept that event.
		 *
		 * @param {tinymce.Editor} ed Editor instance that the plugin is initialized in.
		 * @param {string} url Absolute URL to where the plugin is located.
		 */
		init : function(ed, url) {
			var t=this;
			t.editor = ed;
			
			ed.addCommand('CheckMaxChars', t._checkMaxChars, t);		
			
			t._MCtext;
			ed.onPostRender.add(function(ed,o) { t._MCtext = ed.getContent({format:'text'}).replace(/<\/?[^>]+(>|$)/g, "") }, t);
			ed.onKeyUp.add(t._checkMaxChars, t);
			ed.onChange.add(t._checkMaxChars, t);
			ed.onSetContent.add(t._checkMaxChars, t);
		},

		/**
		 * Returns information about the plugin as a name/value array.
		 * The current keys are longname, author, authorurl, infourl and version.
		 *
		 * @return {Object} Name/value array containing information about the plugin.
		 */
		getInfo : function() {
			return {
				longname : 'Maximum Characters',
				author : 'Lorenzo Campanis',
				authorurl : 'http://www.lcampanis.com',
				infourl : 'http://www.lcampanis.com',
				version : "1.0"
			};
		},
		
		_checkMaxChars : function() {
			var ed = this.editor;
			
			var mc = parseInt(ed.getParam("max_chars"));
			var lb = ed.getParam("max_chars_indicator");
			var cnt_cur = ed.getContent({format:'text'});
			var cnt = cnt_cur.replace(/<\/?[^>]+(>|$)/g, "");	
			
			if( mc > 0 && cnt.length >= 0 && mc<cnt.length && this._MCtext != cnt_cur ) {				
				ed.setContent(this._MCtext);				
			}
			else
				this._MCtext = cnt_cur;
			
			//check for indicator
			if(lb = document.getElementById(lb)) {
				var lb_val = (lb.tagName.toLowerCase() == "input") ? lb.value : parseInt(lb.innerHTML);			
				
				if(lb_val != null) {				
					if(lb.tagName.toLowerCase()=='input')
						lb.value = (mc - cnt.length < 0) ? 0 : mc - cnt.length;
					else
						lb.innerHTML = (mc - cnt.length < 0) ? 0 : mc - cnt.length;
				}
			}
			return 1;
		}
	});

	// Register plugin
	tinymce.PluginManager.add('maxchars', tinymce.plugins.MaxChars);
})();


