﻿<%
'------------------------------------------------------------------------------------------------------------------------------------
' SOOP 2.0 - Phoenix (TNG)
'
' Revision 2.00 - initial 15/02/2008
'
' SOOPPortal All Rights Reserved - Other copyrights as mentioned in sources or joined licences
'------------------------------------------------------------------------------------------------------------------------------------
' Purpose: 
' --------
' This Controller class implements many patterns that aim to streamline the complex and often convoluted workflow of forums mgmt,
' and so pave the way of a solid foundation to extend SooP in further releases. 
' At glance, this class does the following:
' a) process event for a requested activity coming from the outside world (usually via GET param)
' b) set initial (global) page context (i.e. GET/POST params, page-scope params, DB markers) 
' c) perform additional pre-conditions tasks before doing the requested activity (like set/get some attributes before form init / after form post)
' d) do requested activity (one or more "doSomething" methods that are usually calls to INSERT/UPDATE queries)
' e) eventually perform some post-conditions after activity has been done (usually just page redirect)
' f) Ideally, a page layout just reads exposed attributes to render itself properly according context

'------------------------------------------------------------------------------------------------------------------------------------
class class_forum ':: global, class_baseUtils, class_filters, class_DAL, class_io, class_utils, class_languageParser
	'-------------------------------------------------------------------------------------------------------------------------
	' Attributes
	'--------------------------------------------------------------------------------------------------------------------------
	public  fid
	public  fid_n
	public  tid
	public  pid
	public  poid
	public  voteid
	public  fileid
	public  iSort
	public  iView
	public  sTTitle
	public  sBodyn
	public  sSecurityCode
	Public  sSessionName
	public  bCaptcha
	public  page
	public  postsPerPage
	public  sLatest
	public  iOrder1
	public  iOrder2
	public  iOrdertime
	public  datecreated
	public  iqp
	public  iqt
	public  iTType
	public  iExpires
	public  sNotify
	public  aPollAnswers
	public  aPollAnswersPoid
	public  aPoid
	public  iPollAnswersMax	'controls max number of indexed answers allowed in a poll (also add/delete threshold)
	public  sQuote
	public  sQuotedMember
	public  smuser
	public  iTotalVotes
	'
	public  util			'holds class_forum_utils obj
	public  notif			'holds class_forum_notif obj
	'
	private p_sql
	private tid_n
	private pid_n
	private expire
	private remote_addr
	private arrActivity
	private x_i
	'
	'-------------------------------------------------------------------------------------------------------------------------------
	' Init class: get I/O params and set internal environment
	'-------------------------------------------------------------------------------------------------------------------------------
	private sub class_initialize
		set util  = new class_forum_utils
		set notif = new class_forum_notif
		bCaptcha  = true
		iPollAnswersMax = 10
	end sub
	'
	public sub init_io()
		'
		'------------------------------------
		' init attributes from GET
		'------------------------------------
		fid				= cor.filter.checkLng(cor.io.q("fid"))
		if  instr(1, cor.io.q("tid"),"|") <> 0 Then						' composite topicID|postID
			tid		= cor.filter.checkLng(cor.filter.noAnchor(split(cor.io.q("tid"),"|")(0)))
			pid		= cor.filter.checkLng(cor.filter.noAnchor(split(cor.io.q("tid"),"|")(1)))
		else 
			tid		= cor.filter.checkLng(cor.filter.noAnchor(cor.io.q("tid")))	' topicID
			pid		= cor.filter.checkLng(cor.filter.noAnchor(cor.io.q("pid")))	' postID
		end if
		fileid		= cor.filter.checkLng(cor.io.q("fileid"))			' [TODO] file id to delete
		iSort		= cor.filter.checkInt(cor.io.q("sort"))				' [TODO] sort direction of posts
		iView		= cor.filter.checkInt(cor.io.q("view"))				' topic display mode
		page		= cor.filter.checkInt(cor.io.q("pg"))				' forum/topic page
		'
		iOrder1		= cor.filter.checkInt(cor.io.r("order1")) 
		iOrder2		= cor.filter.checkInt(cor.io.r("order2"))
		iOrdertime	= cor.filter.checkInt(cor.io.r("ordertime"))
		'
		iqp			= cor.filter.checkLng(cor.io.q("qp"))
		iqt			= cor.filter.checkLng(cor.io.q("qt"))
		
		poid		= cor.filter.checkLng(cor.io.q("poid"))
		'
		'------------------------------------
		' init attributes from POST 
		'------------------------------------
		if cor_bPostback Then
			fid			= cor.filter.checkLng(cor.io.f("fid"))
			if instr(1, cor.io.f("tid"),"|") <> 0 Then						' composite topicID|postID
				tid		= cor.filter.checkLng(cor.filter.noAnchor(split(cor.io.f("tid"),"|")(0)))
				pid		= cor.filter.checkLng(cor.filter.noAnchor(split(cor.io.f("tid"),"|")(1)))
			else 
				tid		= cor.filter.checkLng(cor.filter.noAnchor(cor.io.f("tid")))	' topicID
				pid		= cor.filter.checkLng(cor.filter.noAnchor(cor.io.f("pid")))	' postID
			end if
			sBodyn			= cor.io.f_rtf("bodyn")
			sSecurityCode	= cor.io.f("securityCode")
			sSessionName	= cor.io.f("sessionName")
			'
			voteid			= cor.filter.checkLng(cor.io.f("voteid"))	' poid ref of a poll vote choice
			'
			sTTitle			= cor.io.f("ttitle")
			iTType			= cor.filter.checkInt(cor.io.f("ttype"))
			iExpires		= cor.filter.checkInt(cor.io.f("expires"))
			sNotify			= lcase(cor.io.f("notify"))
			page			= cor.filter.checkInt(cor.io.f("pg"))
			iqp				= cor.filter.checkLng(cor.io.f("qp"))
			iqt				= cor.filter.checkLng(cor.io.f("qt"))
			fid_n			= cor.filter.checkLng(cor.io.f("tforumid")) 'target fid to move a topic
			'
			aPollAnswers	= getAIO("f", "answer")
			aPollAnswersPoid = 0
			'aPoid			= getAIO("f","")
		end if
		'
		'------------------------------------
		' Global pre-conditions
		'------------------------------------
		'--- read from SooP configuration of how many posts to show per page
		postsPerPage			= cor.filter.checkInt(cor.cfg.item("PostsPerPage")) : if postsPerPage = 0 Then postsPerPage = 15
		'--- get remote address ref
		remote_addr			= request.serverVariables("REMOTE_ADDR")
		'--- add forum language
		cor.lng.includeSection("mod_xmlForum")
	end sub
	'
	public sub init_forum()
		'
		if fid  = 0 Then cor.msg(lngForaForumNotFound)	
		if page = 0 Then page = 1
		'
		' Check access
		call cor.perm.setForumAccess(fid)
		if NOT cor_bRead Then cor.msg(lngNoAccessPermission)
		'
	end sub
	'
	public sub init_topic()
		'
		if tid = 0 Then cor.msg(lngForaTopicNotFound)
		'
		' Forum ID is necessary for topic page to correctly set access permissions (if they're missing it defaults as Guest).
		' To avoid a security breach however, fid is taken from DB and not from IO, as a malicious user could
		' try to tamper a fid with more privilegues to gain access to a restricted topic  
		fid = cor.dal.getvalue("SELECT t.tforumid FROM " & cor_dbpre & "topics AS t WHERE t.tid = {0}", array(tid))
		'
		' Check access
		call cor.perm.setForumAccess(fid)
		if NOT cor_bRead Then cor.msg(lngNoAccessPermission)
		'
		'#<block> topic paging related
		'
		'--- set (persistent) topic display mode
		if iView <> 0 Then session(cor_pre&"viewmode") = iView else iView = cor.filter.checkInt(session(cor_pre&"viewmode"))
		if iView  = 0 Then iView = cor_cViewAsList
		'
		'--- determine topic page to display
		if page   = 0 Then page  = findTopicPage()
		'
		'--- paging is always enabled in listview mode
		if page  <> 1 Then iView = cor_cViewAsList
		'
		'#</block> topic paging related
		'
		'---  We shall display captcha only for unauthenticated users
		if (cor.filter.checkInt(cor.cfg("quickreply")) > 0 AND cor_bGuest) Then bCaptcha = true else bCaptcha = false
		'
		'--- Update how many times current topic has been seen
		if  clng(session(cor_pre&"lastpost")) <> tid Then
			cor.dal.exec "UPDATE " & cor_dbpre & "topics SET tseen=tseen+1 WHERE tid = {0}", array(tid)
			session(cor_pre&"lastpost") = tid
		end if
		'
	end sub
	'
	public sub init_postform()
		'
		' Ensure a fid always exists
		if tid <> 0 Then'AND fid = 0 
			p_sql = "SELECT t.tforumid FROM " & cor_dbpre & "topics AS t WHERE t.tid ={0}"
			fid = cor.filter.checkInt(cor.dal.getValue(p_sql, array(tid)))
		end if
		'
		' Check access
		if fid = 0 Then cor.redir("fora_start.asp")
		call cor.perm.setForumAccess(fid)
		if NOT cor_bRead Then cor.redir("fora_start.asp")
		'
		if (cor.filter.checkInt(cor.cfg("topic")) > 0 AND cor_bGuest) Then bCaptcha = true else bCaptcha = false
		datecreated = cor.butil.cvUxTime(Now())	
		'
	end sub
	'
	'------------------------------------------------------------------------------------------------------------------------------------------
	'
	' Event Process (map inbound events to activities)
	'
	'------------------------------------------------------------------------------------------------------------------------------------------
	public sub controller(byVal eventIn)
		dim iDo : iDo = cor.filter.checkInt(eventIn) : if iDo = 0 Then exit sub
		'
		select case iDo
			'
		case cor_cDoDeleteTopic
			'
			cor.butil.checkNotVoid("[cor.forum:cor_cDoDeleteTopic],fid,tid")
			doDeleteTopic()
			cor.redir("fora_forum.asp?fid="& fid)
			'
		case cor_cDoDeletePost
			'
			cor.butil.checkNotVoid("[cor.forum:cor_cDoDeletePost],fid,tid,pid")
			doDeletePost()
			cor.redir("fora_topic.asp?tid=" & tid & "&pg=" & page)
			'
		case cor_cDoDeleteFile
			'
			cor.butil.checkNotVoid("[cor.forum:cor_cDoDeleteFile],fileid,tid")
			doDeleteFile()
			cor.redir("fora_topic.asp?tid=" & tid) 
			'
		case cor_cDoDeletePollAnswer
			'
			cor.butil.checkNotVoid("[cor.forum:cor_cDoDeletePollAnswer],fid,poid,tid")
			doDeletePollAnswer()
			cor.redir("fora_postform.asp?do=33&amp;fid=" & fid &"&amp;tid=" & tid)
			'
		case cor_cDoLockTopic
			'
			cor.butil.checkNotVoid("[cor.forum:cor_cDoLockTopic],tid")
			doLockTopic()
			cor.redirm lngForaTopicLocked, "fora_topic.asp?tid=" & tid
			'
		case cor_cDoUnlockTopic
			'
			cor.butil.checkNotVoid("[cor.forum:cor_cDoUnlockTopic],tid")
			doUnlockTopic()
			cor.redirm lngForaTopicUnLocked, "fora_topic.asp?tid=" & tid
			'
		case cor_cDoMakeSticky
			'
			cor.butil.checkNotVoid("[cor.forum:cor_cDoMakeSticky],tid")
			doMakeSticky()
			cor.redirm lngForaTopicSticky, "fora_topic.asp?tid=" & tid
			'
		case cor_cDoRemoveSticky
			'
			cor.butil.checkNotVoid("[cor.forum:cor_cDoRemoveSticky],tid")
			doRemoveSticky()
			cor.redirm lngForaTopicUnSticky, "fora_topic.asp?tid="& tid
			'
		case cor_cDoSubscribeForum
			'
			cor.butil.checkNotVoid("[cor.forum:cor_cDoSubscribeForum],fid")
			doSubscribeForum()
			cor.redirm lngForaSubscriptionadded, "fora_forum.asp?fid="& fid
			'
		case cor_cDoUnsubscribeForum
			'
			cor.butil.checkNotVoid("[cor.forum:cor_cDoUnsubscribeForum],fid")
			doUnsubscribeForum()
			cor.redirm lngForaSubscriptiondeleted, "fora_forum.asp?fid="& fid
			'
		case cor_cDoSubscribeTopic
			'
			cor.butil.checkNotVoid("[cor.forum:cor_cDoSubscribeTopic],fid,tid")
			doSubscribeTopic()
			cor.redirm lngForaSubscriptionaddedtopic, "fora_topic.asp?tid="& tid
			'
		case cor_cDoUnsubscribeTopic
			'
			cor.butil.checkNotVoid("[cor.forum:cor_cDoUnsubscribeTopic],fid,tid")
			doUnsubscribeTopic()
			cor.redirm lngForaSubscriptiondeletedtopic, "fora_topic.asp?tid="& tid
			'
		case cor_cDoNewPoll
			'
			' init/post form
			'
			if iExpires = 0 then
				expire = cor.butil.cvUxTime(dateAdd("yyyy",10,Now()))
			else
				expire = cor.butil.cvUxTime(dateAdd("d",iExpires,Now()))
			end if
			' init array only during GET, with number of answers to display (default=1)
			if NOT isarray(aPollAnswers) Then redim aPollAnswers(0) 
			'
			' post form
			'
			if NOT cor_bPostback Then
				exit sub
			else
'				cor.str.wri "<script type=""text/javascript"">$('label[class^=errorfrm error]').remove();</script>"
'				cor.butil.checkNotVoid("[cor.forum:cor_cDoNewPoll-POST],fid")
				if len(validateForum(bCaptcha))>0 then
					cor.str.wri validateForum(bCaptcha)
					call cor.butil.errorMsg(lngErrorDisplayLine1,lngValidationFail,True,3000,false,,0,false)
				else
					tid_n = doNewPoll()
					'cor.str.wri "<script type=""text/javascript"">window.location='fora_topic.asp?tid=" & tid_n & "';</script>"
					'cor.redir("fora_topic.asp?tid="& tid_n)
					call cor.butil.successMsg(lngSuccessfullPost,lngSystemMessage,True,3000,true,"fora_topic.asp?tid="& tid_n,3500,false)
				end if
				response.end
			end if
			
			'
		case cor_cDoNewTopic
			if NOT cor_bPostback Then
				exit sub
			else' post form
'				cor.str.wri "<script type=""text/javascript"">$('label[class^=errorfrm error]').remove();</script>"
'				cor.butil.checkNotVoid("[cor.forum:cor_cDoNewTopic-POST],fid,sTTitle")
				if len(validateForum(bCaptcha))>0 then
					cor.str.wri validateForum(bCaptcha)
					call cor.butil.errorMsg(lngErrorDisplayLine1,lngValidationFail,True,3000,false,,0,false)
				else
					tid_n = doNewTopic(0)
					'cor.str.wri "<script type=""text/javascript"">window.location='fora_topic.asp?tid=" & tid_n & "';</script>"
					call cor.butil.successMsg(lngSuccessfullPost,lngSystemMessage,True,3000,true,"fora_topic.asp?tid="& tid_n,3500,false)
					'cor.redir("fora_topic.asp?tid="& tid_n)
				end if
				response.end
			end if
			'
		case cor_cDoNewPost

			if NOT cor_bPostback Then
				exit sub
			Else
				DoPostForm()
			End If

		case cor_cDoNewPostExt
			'
			' post form
			'
			if NOT cor_bPostback Then
				exit sub
			else
				DoPostForm()
			end if
			'
		case cor_cDoNewPostExtQ 'New Post with Quote
			'
			if NOT cor_bPostback Then
				'
				' init form
				'
				cor.butil.checkNotVoid("[cor.forum:cor_cDoNewPostExtQ-INIT],fid,tid")
				p_sql = "SELECT t.ttitle, t.tbody FROM " & cor_dbpre & "topics AS t WHERE t.tid={0}"
				cor.dal.open arrActivity, p_sql, array(tid)
				if cor.butil.eof(arrActivity) then cor.msg("[cor_cDoNewPost-INIT] Topic ID "&tid&" was not found")
				'
				' quoted topic
				'
				if iqt > 0 then '<qt> 
					'--------------- 0 ------ 1 ------ 2 ---------
					p_sql = "SELECT m.muser, t.tbody, t.ttitle FROM " & cor_dbpre & "topics AS t " & _
					"INNER JOIN " & cor_dbpre & "members AS m ON (m.MemberID = t.tmemberid) WHERE t.tid ={0}"
					cor.dal.open arrActivity, p_sql, array(iqt)
					if cor.butil.eof(arrActivity) then cor.msg("[cor_cDoNewPostExtQ-INIT] " & lngSysTopicId & tid & lngSysWasNotFound)
					if cor_bUseRichEditor Then
						sQuote = "<blockquote class=""quotetop"">" & arrActivity(0,0) & " - " & arrActivity(2,0) & "</blockquote><blockquote class=""quotemain"">" & arrActivity(1,0) & "</blockquote>" & vbcrlf &"<p></p>"
					else
						sQuote = arrQuote(1,0)
					end if
					sQuotedMember = arrActivity(0,0)
				end if '</qt>
				'
				' quoted post
				'
				if iqp > 0 then '<qp>
					'--------------- 0 ------ 1 -----------
					p_sql = "SELECT m.muser, p.pbody FROM " & cor_dbpre & "posts AS p " _
						  & "INNER JOIN " & cor_dbpre & "members AS m ON (m.MemberID = p.pmemberid) WHERE p.pid ={0}"
					cor.dal.open arrActivity, p_sql, array(iqp)
					if cor.butil.eof(arrActivity) then cor.msg("[cor_cDoNewPostExtQ-INIT] " & lngSysCantFindPostId & pid)
					if cor_bUseRichEditor Then
						sQuote = "<blockquote class=""quotetop"">" & arrActivity(0,0) & "</blockquote><blockquote class=""quotemain"">" & arrActivity(1,0) & "</blockquote>" & vbcrlf &"<p></p>"
					else
						sQuote = arrActivity(1,0)
					end if
					sQuotedMember = arrActivity(0,0)
				end if '</qp>
				'
				' add quote as default editor content
				sBodyn = sQuote
			else

			if NOT cor_bPostback Then
				exit sub
			Else
				DoPostForm()
			End If

			end if
			'
		case cor_cDoEditTopic
			'
			' pre-conditions
			'
			if NOT cor_bPostback Then
				'
				'--- init form
				' 
				cor.butil.checkNotVoid("[cor.forum:cor_cDoEditTopic-INIT],tid")
				'--------------- 0 ------- 1 ------ 2 --------- 3 --------4-----
				p_sql =	"SELECT t.ttitle, t.tbody, t.tforumid, m.muser, t.ttype " _
					  & "FROM "   & cor_dbpre & "topics AS t INNER JOIN " & cor_dbpre & "members AS m ON (m.MemberID = t.tmemberid) " & _
					  " WHERE t.tid ={0}"
				if  NOT cor_bModerator Then
					p_sql = p_sql & " AND "   & cor_dbpre & "t.tmemberid ={1}"
				end if
				cor.dal.open arrActivity, p_sql, array(tid, cor_lUserID)
				if cor.butil.eof(arrActivity) then cor.msg("[cor_cDoEditTopic-INIT] " & lngSysInvalidTopicId) 
				'
				sTTitle	= cor.butil.htmlEncode(arrActivity(0,0))
				sBodyn	= arrActivity(1,0)
				iTType	= arrActivity(4,0) 
				fid		= arrActivity(2,0)	' as fora_topic.asp link doesn't give one, ensure a fid gets populated for postback
				smuser	= arrActivity(3,0)
				'
			else
				'
				' post form
				'
'				cor.str.wri "<script type=""text/javascript"">$('label[class^=errorfrm error]').remove();</script>"
				cor.butil.checkNotVoid("[cor.forum:cor_cDoEditTopic-POST],fid,tid,fid_n,sTTitle")
				
				if len(validateForum(bCaptcha))>0 then
					cor.str.wri validateForum
					call cor.butil.errorMsg(lngErrorDisplayLine1,lngValidationFail,True,3000,false,,0,false)
					response.end
				else
					if fid = fid_n then '--- topic was only edited
						doEditTopic()
'						cor.str.wri "<script type=""text/javascript"">window.location='fora_topic.asp?tid=" & tid & "|" & pid & "#a" & pid &"';</script>"
						call cor.butil.successMsg(lngSuccessfullPost,"Validation Successful",True,3000,true,"fora_topic.asp?tid=" & tid & "|" & pid & "#a" & pid,3500,false)
					else '--- topic was moved (and maybe edited) to another forum
						doEditTopic()
						tid_n = doMoveTopic(fid_n)
						'cor.str.wri "<script type=""text/javascript"">window.location='fora_topic.asp?tid=" & tid_n & "';</script>"
						call cor.butil.successMsg(lngSuccessfullPost,"Validation Successful",True,3000,true,"fora_topic.asp?tid="&tid_n,3500,false)
						'cor.redir("fora_topic.asp?tid="&tid_n)
					end if
					response.end
				end if
			end if
			'
		case cor_cDoEditPost
			'
			' pre-conditions
			'
			if NOT cor_bPostback Then
				'
				'--- init form
				'charles dropped p.pmember, added m.muser
				cor.butil.checkNotVoid("[cor.forum:cor_cDoEditPost-INIT],tid,pid")
				'--------------- 0 ------- 1 -------- 2 ----------
				p_sql = "SELECT p.pbody, p.pforumid, m.muser " _
					  &	"FROM "   & cor_dbpre & "posts AS p INNER JOIN " & cor_dbpre & "members AS m ON (m.MemberID = p.pmemberid) " & _
					  "WHERE p.pid ={0}"
				if NOT cor_bModerator Then
					p_sql = p_sql & " AND p.pmemberid ={1}"
				end if
				cor.dal.open arrActivity, p_sql, array(pid, cor_lUserID)
				if cor.butil.eof(arrActivity) then cor.msg("[cor_cDoEditPost-INIT] " & lngSysInvalidPostId) 
				'
				sTTitle	= ""' not used in posts
				sBodyn	= arrActivity(0,0)
				fid		= arrActivity(1,0)	' as fora_topic.asp link doesn't give one, ensure a fid gets populated for postback
				smuser	= arrActivity(2,0)
				'
			else
				'
				' post form
				cor.str.wri "<script type=""text/javascript"">$(document).ready(function(){$('label[class^=errorfrm error]').remove();});</script>"
				cor.butil.checkNotVoid("[cor.forum:cor_cDoEditPost-POST],fid,tid,pid")
				
				if len(validateForum(bCaptcha))>0 then
					cor.str.wri validateForum(bCaptcha)
					call cor.butil.errorMsg(lngErrorDisplayLine1,lngValidationFail,True,3000,false,,0,false)
				else
					doEditPost()
					'cor.str.wri "<script type=""text/javascript"">window.location='fora_topic.asp?tid=" & tid & "|" & pid & "&pg=" & page & "#a" & pid & "';</script>"
					call cor.butil.successMsg(lngSuccessfullPost,lngSystemMessage,True,3000,true,"fora_topic.asp?tid=" & tid & "|" & pid & "&pg=" & page & "#a" & pid,3500,false)
					'cor.redir("fora_topic.asp?tid=" & tid & "|" & pid & "&pg=" & page & "#a" & pid)
				end if
				response.end
			end if
			'
		case cor_cDoEditPoll
			'
			' pre-conditions
			'
			if NOT cor_bPostback Then
				'
				' init form
				'
				'Dim aPollAnswersPoid
				cor.butil.checkNotVoid("[cor.forum:cor_cDoEditPoll-INIT],tid")
				'-----------------0------1---------2--------3---------4---
				p_sql = "SELECT t.tid, t.ttitle, t.tbody, t.ttype, po.poid FROM " & cor_dbpre & "topics AS t" _
					& " LEFT JOIN " & cor_dbpre & "polls AS po ON po.potopicid=t.tid" _
					& " WHERE t.tid = {0}"
				cor.dal.open arrActivity, p_sql, array(tid)
				if cor.butil.eof(arrActivity) then cor.msg("[ERR:cor_cDoEditPoll-INIT] " & lngSysTopicId & tid & lngSysWasNotFound)
				'
				tid		= arrActivity(0,0)
				sTTitle	= arrActivity(1,0)
				sBodyn	= arrActivity(2,0)
				iTType	= arrActivity(3,0)
				poid	= arrActivity(4,0)
				'
				'---------------- 0 ------ 1 ---------- 2 ---------- 3 ---------------
				p_sql = "SELECT po.poid, po.poanswer, po.popollid, po.poexpires FROM " _
					  & cor_dbpre & "polls AS po WHERE po.poisquestion IS NULL AND po.potopicid = {0} ORDER BY po.poid"
				cor.dal.open arrActivity, p_sql, array(tid)'
				if cor.butil.eof(arrActivity) then cor.msg("[ERR:cor_cDoEditPoll-INIT] "& lngSysPollTopicId & tid & lngSysWasNotFound)
				'
				redim aPollAnswers(ubound(arrActivity,2))
				redim aPollAnswersPoid(ubound(arrActivity,2))
				' expose answers array for chosen topic
				for x_i = 0 to ubound(arrActivity,2)
					aPollAnswersPoid(x_i)	= arrActivity(0,x_i)
					aPollAnswers(x_i)		= arrActivity(1,x_i)
				next
				'
			else
				'
				' post form
				'
				cor.butil.checkNotVoid("[cor.forum:cor_cDoEditPoll-POST],tid,sTTitle")

				If len(validateForum(bCaptcha))>0 Then
					cor.str.wri validateForum(bCaptcha)
					call cor.butil.errorMsg(lngErrorDisplayLine1,lngValidationFail,True,3000,false,,0,false)
				Else
					doEditPoll()
					'cor.str.wri "<script type=""text/javascript"">window.location='fora_topic.asp?tid=" & tid & "';</script>"
					call cor.butil.successMsg(lngSuccessfullPost,lngSystemMessage,True,3000,true,"fora_topic.asp?tid=" & tid,3500,false)
					'cor.redir("fora_topic.asp?tid=" & tid)
				End If
				response.end
			end if
			'
		case cor_cDoGoNextTopic
			'
			cor.butil.checkNotVoid("[cor.forum:cor_cDoGoNextTopic],fid,tid")
			tid_n = doGoNextTopic()
			cor.redir("fora_topic.asp?tid=" & tid_n)
			'
		case cor_cDoGoPrevTopic
			cor.butil.checkNotVoid("[cor.forum:cor_cDoGoPrevTopic],fid,tid")
			tid_n = doGoPrevTopic()
			cor.redir("fora_topic.asp?tid=" & tid_n)
			'
		case cor_cDoPollVote
			'
			' post form
			'
			if NOT cor_bPostback Then exit sub
			cor.butil.checkNotVoid("[cor.forum:cor_cDoPollVote-POST],tid,voteid")
			cor_cDoPollVote(voteid)
			cor.redir("fora_topic.asp?tid="& tid)
			'
		case cor_cDoPollScore
			' just a page view marker. Nothing to do
		case else
			 cor.msg lngUnknownActivity
		end select
		'
	end sub 
	'
	'--------------------------------------------------------------------------------------------------------------------------------
	'
	' Forum Activities
	'
	'-----------------------------------------------------------------------------------------------------------------------
	private sub doDeleteTopic()
		dim arrDelTopic, postsInTopic, x_i
		'
		'--- pay attention to the special fictive post for counters update!
		cor.dal.open arrDelTopic,"SELECT t.tmemberid FROM " & cor_dbpre & "topics AS t WHERE t.tid = {0}", array(tid)
		if NOT cor.butil.eof(arrDelTopic) Then call updatePostsByUser(arrDelTopic(0,0),-1)
		'
		postsInTopic = cor.dal.getValue("SELECT COUNT(*) FROM " & cor_dbpre & "posts AS p WHERE p.ptopicid = {0} AND p.pistopic = 0", array(tid))
		if  len(postsInTopic) = 0 Then
			postsInTopic = 0
		else
			postsInTopic = clng(postsInTopic)
		end if
		cor.dal.exec "UPDATE " & cor_dbpre & "forums SET sposts = sposts - {0}, stopics = stopics - 1 WHERE sid = {1}", array(postsInTopic, fid)
		'
		'--- for each post we will decrement the concerned members posts counter
		cor.dal.open arrDelTopic, "SELECT p.pmemberid FROM " & cor_dbpre & "posts AS p WHERE p.pistopic = 0 AND p.ptopicid = {0}", array(tid)
		if  NOT cor.butil.eof(arrDelTopic) Then
			for x_i=0 To ubound(arrDelTopic,2)
				call updatePostsByUser(arrDelTopic(0,x_i), -1)
			next
		end if
		'we are deleting based on Topics.tid OR Topics.newtid.  Add the newtid to the Notify delete.
		cor.dal.exec "DELETE FROM " & cor_dbpre & "notify WHERE ntopicid = {0} OR ntopicid IN " & _
						"(SELECT tid FROM " & cor_dbpre & "topics WHERE newtid = {0})", array(tid)
		'Delete posts.
		cor.dal.exec "DELETE from " & cor_dbpre & "posts WHERE ptopicid = {0}", array(tid)
		
		'
		'--- if poll also delete the poll and its questions and the related vote!
		cor.dal.open arrDelTopic,"SELECT t.tid, t.tpoll FROM " & cor_dbpre & "topics AS t WHERE t.tid = {0}", array(tid)
		if  NOT cor.butil.eof(arrDelTopic) Then
			if  cor.filter.checkInt(arrDelTopic(1,0)) = 1 Then
				cor.dal.exec "DELETE FROM " & cor_dbpre & "pollvotes WHERE vtopicid = {0}", array(tid)
				cor.dal.exec "DELETE FROM " & cor_dbpre & "polls WHERE potopicid = {0} AND poisquestion IS NULL", array(tid)
				cor.dal.exec "DELETE FROM " & cor_dbpre & "polls WHERE potopicid = {0} ", array(tid)
			end if
		end if	
		cor.dal.exec "DELETE FROM " & cor_dbpre & "topics WHERE tid = {0} OR newtid= {0} ", array(tid)
		'
		call updateLastPost( fid, tid )
		call updatePostsByUser(null,null)
		'
		call cor.audit.writeLogDb(8,"Topic (and related posts) " & tid & " was deleted")
		'
	end sub
	'
	private sub doDeletePost()
		dim arrDelPost, intLastPost, intUserId
		'	
		p_sql = "SELECT p.pattach, p.pmemberid FROM " & cor_dbpre & "posts AS p WHERE p.pid= {0}"
		cor.dal.open arrDelPost, p_sql, array(pid)
		'
		if NOT cor.butil.eof(arrDelPost) Then call updatePostsByUser( arrDelPost(1,0),-1 )
		'
		cor.dal.exec "DELETE FROM " & cor_dbpre & "posts WHERE " & cor_dbpre & "posts.pid = {0}", array(pid)
		'
		cor.dal.exec "UPDATE "		& cor_dbpre & "topics SET tanswers = tanswers - 1 WHERE tid = {0}", array(tid)
		cor.dal.exec "UPDATE "		& cor_dbpre & "forums SET sposts = sposts - 1 WHERE sid = {0}", array(fid)
		intLastPost = cor.dal.getValue("SELECT COUNT(*) FROM "	 & cor_dbpre & "posts AS p WHERE p.ptopicid = {0}", array(tid))
		if  clng(intLastPost) = 0 Then
			cor.dal.exec "UPDATE "  & cor_dbpre & "topics SET tdatelast = tdatecreate WHERE tid = {0}", array(tid)
		else
			intLastPost = cor.dal.getValue("SELECT MAX(pdatecreate) FROM " & cor_dbpre & "posts AS p WHERE p.ptopicid = {0}", array(tid))
			cor.dal.exec "UPDATE "  & cor_dbpre & "topics SET tdatelast = {0} WHERE tid = {1}", array(clng(intLastPost),tid)
		end if
		'
		call updateLastPost( fid,tid )
		call updatePostsByUser(null,null)
		'
		call cor.audit.writeLogDb(8,"Post " &  pid & " Was deleted")
	end sub
	'
	private sub doDeleteFile()
'		dim arrDownload
'
'		p_sql = "SELECT dl.did, dl.durl FROM " & cor_dbpre & "downloads AS dl WHERE dl.did = {0}"
'		cor.dal.open arrDownload, p_sql, array(fileid)
'		if cor.butil.eof(arrDownload) Then exit sub
'		call cor.obj.init("fso")
'		if   cor.obj.fso.fileExists (server.mappath(arrDownload(1,0))) Then
'			 cor.obj.fso.deleteFile  server.mappath(arrDownload(1,0)), true
'		end if
'		' delete ALL references to the file being deleted
'		cor.dal.exec "DELETE FROM " & cor_dbpre & "downloads WHERE did = {0}",					array(arrDownload(0,0))
'		cor.dal.exec "UPDATE "	  & cor_dbpre & "topics SET tattach = 0 WHERE tattach = {0}", array(arrDownload(0,0))
'		cor.dal.exec "UPDATE "	  & cor_dbpre & "posts SET pattach = 0 WHERE pattach = {0}",  array(arrDownload(0,0))
	end sub
	'
	private sub doDeletePollAnswer()
		
		cor.dal.exec "DELETE FROM " & cor_dbpre & "polls WHERE " & cor_dbpre & "polls.poid = {0}", array(poid)
		'
		call cor.audit.writeLogDb(38,"Poll Answer """ &  poid & """ Was deleted")
		cor.redir("fora_postform.asp?do=33&amp;tid="&cor.io.q("tid"))
	end sub
	'
	private sub doLockTopic()
		cor.dal.exec "UPDATE " & cor_dbpre & "topics SET tlocked = 1 WHERE tid ={0}", array(tid)
		call cor.audit.writeLogDb(16,"Locked topic " & tid)
	end sub
	'
	private sub doUnlockTopic()
		cor.dal.exec "UPDATE " & cor_dbpre & "topics SET tlocked = 0 WHERE tid ={0}", array(tid)
		call cor.audit.writeLogDb(16,"Unlocked topic " &  tid)
	end sub
	'
	private sub doMakeSticky()
		cor.dal.exec "UPDATE " & cor_dbpre & "topics SET ttype = 2 WHERE tid = {0}", array(tid)
		call cor.audit.writeLogDb(17,"Topic " &  tid & " Was SET to sticky")
	end sub
	'
	private sub doRemoveSticky()
		cor.dal.exec "UPDATE " & cor_dbpre & "topics SET ttype = 0 WHERE tid = {0}", array(tid)
		call cor.audit.writeLogDb(17,"Removed sticky from topic " &  tid)
	end sub
	'
	private sub doSubscribeForum()
		cor.forum.notif.addnotify fid,   0, cor_lUserID
	end sub
	'
	private sub doUnsubscribeForum()
		'--- also deletes all topics (un)subscriptions specific to this forum
		cor.forum.notif.delnotify fid,   0, cor_lUserID
	end sub
	'
	private sub doSubscribeTopic()
		cor.forum.notif.addnotify fid, tid, cor_lUserID
	end sub
	'
	private sub doUnsubscribeTopic()
		cor.forum.notif.delnotify fid, tid, cor_lUserID
	end sub
	'
	private function doNewPoll()
		dim maxtid, maxpoid, x
		'
		' A poll resides in a poll-enabled topic (tpoll=1)
		maxtid  = doNewTopic(1)
		'
		sTTitle = cor.filter.BadWordFilter(sTTitle)
		'response.write "INSERT INTO " & cor_dbpre & "polls (poquestion,poisquestion,poexpires,potopicid) VALUES ({0},{1},{2},{3})"
		cor.dal.exec "INSERT INTO " & cor_dbpre & "polls (poquestion,poisquestion,poexpires,potopicid) VALUES ({0},{1},{2},{3})", array(sTTitle, 1, expire, maxtid)
		maxpoid = clng(cor.dal.getValue("SELECT MAX(po.poid) FROM " & cor_dbpre & "polls AS po", empty))
		for x = 0 to ubound(aPollAnswers)
			cor.dal.exec "INSERT INTO " & cor_dbpre & "polls (poanswer,popollid,potopicid) VALUES ({0},{1},{2})", array(aPollAnswers(x), maxpoid, maxtid)
		next
		cor.dal.exec "UPDATE " & cor_dbpre & "topics SET tshow = 1 WHERE tid ={0}", array(maxtid)
		'
		doNewPoll = maxtid
	end function
	'
	private function doNewTopic(tpoll)
		dim maxtid
			' 0		1	2 			3		4			5			6			7	8	9	10		11		12	13		14		15
			p_sql = "INSERT INTO " & cor_dbpre & "topics " _
				& "(tbody,ttitle,tmemberid,tforumid,tdatecreate,tdatechanged,tdatelast,tip,tshow,tpoll,tseen,newtid,ttype,tlocked,tanswers,tlastmid) " _
				& "VALUES ({0},{1},{2},{3},{4},{5},{6},{7},{8},{9},{10},{11},{12},{13},{14},{15})"
			sBodyn = cor.filter.BadWordFilter(sBodyn)
			sTTitle = cor.filter.BadWordFilter(sTTitle)
			cor.dal.exec p_sql, array(sBodyn, sTTitle, cor_lUserID, fid, datecreated, datecreated, datecreated, remote_addr, 1, tpoll, 0, 0, iTType, 0, 0, cor_lUserID)
		'Added a WHERE clause here, to make sure we get the right tid.  Otherwise, near-simultaneous inserts could corrupt this.
		if isnumeric(sTTitle) then
			maxtid = clng(cor.dal.getValue("SELECT MAX(t.tid) FROM " & cor_dbpre & "topics AS t WHERE t.ttitle ='{0}'", array(sTTitle)))
		else
			maxtid = clng(cor.dal.getValue("SELECT MAX(t.tid) FROM " & cor_dbpre & "topics AS t WHERE t.ttitle ={0}", array(sTTitle)))
		end if
		'
		' create the fictive corresponding post!!!
		'charles Dropped pmember (before pmemberid). cor_sLoggedinusername
		p_sql = "INSERT INTO " & cor_dbpre & "posts " _
			& "(pbody,pmemberid,pforumid,pdatecreate,pdatechanged,pip,pistopic,ptopicid) " _
			& "VALUES ({0},{1},{2},{3},{4},{5},{6},{7})"
		cor.dal.exec p_sql, array(sBodyn, cor_lUserID, fid, datecreated, datecreated, remote_addr, 1, maxtid)
		' 
		' Update forums data
		cor.dal.exec "UPDATE " & cor_dbpre & "forums SET stopics = stopics + 1 WHERE sid = {0}", array(fid)
		'
		call updateLastPost(fid, maxtid)
		call updatePostsByUser(cor_lUserID, 1)
		'
		' Check if user wants email notifications
		if  sNotify = "on" then
			cor.forum.notif.addnotify  fid, maxtid, cor_lUserID 
		else
			cor.forum.notif.delnotify  fid, maxtid, cor_lUserID 
		end if
		'
		'  notify all other forum subscribers
		cor.forum.notif.sendnotify fid, maxtid,  "newtopic"
		'
		session(cor_pre&"lastpost") = maxtid
		'
		doNewTopic = maxtid
	end function
	'
	private function doNewPost()
		dim maxpid
		'
		'charles dropped pmember (before pmemberid) and cor_sloggedinusername
		p_sql = "INSERT INTO " & cor_dbpre & "posts " _
			  & "(pbody,ptopicid,pdatecreate,pmemberid,pforumid,pip,pistopic,pdatechanged) " _
			  & "VALUES ({0},{1},{2},{3},{4},{5},{6},{7})"
		sBodyn = cor.filter.BadWordFilter(sBodyn)
		cor.dal.exec p_sql, array(sBodyn, tid, cor.butil.cvUxTime(Now()),  cor_lUserID, fid, remote_addr, 0, cor.butil.cvUxTime(Now()))
		maxpid = clng(cor.dal.getValue("SELECT MAX(p.pid) FROM " & cor_dbpre & "posts AS p ", empty))
		cor.dal.exec "UPDATE " & cor_dbpre & "topics SET tanswers = tanswers + 1,tdatelast = {0} WHERE tid = {1}", array(cor.butil.cvUxTime(Now()), tid)
		cor.dal.exec "UPDATE " & cor_dbpre & "forums SET sposts = sposts + 1 WHERE sid = {0}", array(fid)
		'
		call updateLastPost(fid, tid)
		call updatePostsByUser(cor_lUserID, 1)
		'
		' if member has subscribed to this forum or topic, reset nseen=0 to allow sending him e-mail notifications
		cor.forum.notif.resetnotify fid, tid, cor_lUserID
		'
		' notify all other topic subscribers
		cor.forum.notif.sendnotify fid, tid, "newpost"
		'
		doNewPost = maxpid
	end function
	'
	private function doMoveTopic(tforumid)
		dim arrMove, maxtid, sBodynMoved, isPoll
		'
		' Get all topic data
		'--------------- 0 ------------ 1 -------- 2 ---------- 3 ---------- 4 --------- 5 ------ 6 -------- 7 --------- 8 -------9------10--------11---
		p_sql = "SELECT t.tdatecreate, m.muser, t.tmemberid, t.tdatelast, t.tanswers, t.tseen, t.tattach, t.tforumid, t.tlocked, t.tip, t.tpoll, t.ttype " _
			  &	"FROM " & cor_dbpre & "topics AS t INNER JOIN " & cor_dbpre & "members AS m ON (m.MemberID = t.tmemberid) WHERE t.tid = {0}"

		cor.dal.open arrMove, p_sql, array(tid)
		if cor.butil.eof(arrMove) then cor.redir("[ERR:doMoveTopic] can't find Topic ID "&tid)
		if arrMove(10,0) > 0 then
			isPoll = 1
		else
			isPoll = 0
		end if
		'
		' Create a topic copy that points to the new tforumid
		p_sql = "INSERT INTO " & cor_dbpre & "topics (tbody,ttitle,tdatechanged,tforumid,tdatecreate,tmemberid, tdatelast,tshow,tanswers,tseen,tattach,tlocked,tlastmid,oldtid, tip, tpoll, ttype, newtid) " _
			  & "VALUES ({0},{1},{2},{3},{4},{5},{6},{7},{8},{9},{10},{11},{12},{13},{14},{15},{16},{17})"
		' -------------------------tbody--ttitle---tdatechanged------------------------tforumid--tdatecreate---tmemberid----tdatelast----tshow--tanswers-----tseen----------tattach------tlocked) 
		'problems crop up if the body or the title is only numeric.
		if isnumeric(sTTitle) then	p_sql = replace(p_sql, "{1},", "'{1}',")
		if isnumeric(sBodyn) then	P_sql = replace(p_sql, "{0},", "'{0}',")
		'problems crop up with nulls that somehow magically transform into ''.
		if isempty(arrMove(4,0)) or isnull(arrMove(4,0)) or len(arrMove(4,0)) < 1 then	arrMove(4,0) = 0
		if isempty(arrMove(6,0)) or isnull(arrMove(6,0)) or len(arrMove(6,0)) < 1 then	arrMove(6,0) = 0
		cor.dal.exec p_sql, array(sBodyn, sTTitle, cor.butil.cvUxTime(Now()), tforumid, arrMove(0,0), arrMove(2,0), arrMove(3,0), 1, arrMove(4,0), arrMove(5,0), arrMove(6,0), arrMove(8,0),cor_lUserID,tid,arrMove(9,0), isPoll, arrMove(11,0),0)
		'maxtid = clng(cor.dal.getValue("SELECT MAX(t.tid) FROM " & cor_dbpre & "topics AS t ", empty))
		'make sure we get the RIGHT maxtid.  Unlikely, but possible that someone would create a new topic at the moment we copy this moved topic.
		maxtid = clng(cor.dal.getValue("SELECT MAX(t.tid) FROM " & cor_dbpre & "topics AS t WHERE oldtid = {0}", array(tid)))
		'
		' Lock old topic and update references, so visitor shall find moved topic/thread when referenced in previous forum 
		' 1) tmoved = new fid 2) newtid = new tid
		sBodynMoved = "<p><i><a href=""fora_topic.asp?tid=" & maxtid & """>" & lngForaMovedTopic & "</a></i></p>"
		p_sql =  "UPDATE " & cor_dbpre & "topics SET tbody={0}, tdatechanged={1}, tmoved = {2}, newtid = {3}, tlocked={4} WHERE tid = {5}"
		cor.dal.exec p_sql , array(sBodynMoved, cor.butil.cvUxTime(Now()), tforumid, maxtid, 1, tid)
		'
		' Point old posts to the new forum and topic
		p_sql = "UPDATE " & cor_dbpre & "posts SET pforumid={0}, ptopicid={1} WHERE ptopicid={2}"
					
		cor.dal.exec p_sql, array(tforumid, maxtid, tid)
		'
		'Poll?
		
		if isPoll = 1 then				'Move polls as well.
			p_sql = "UPDATE " & cor_dbpre & "polls SET potopicid = {0} WHERE potopicid = {1}"
			cor.dal.exec p_sql, array(maxtid, tid)

			p_sql = "UPDATE " & cor_dbpre & "pollvotes SET vtopicid = {0} WHERE vtopicid = {1}"
			
			cor.dal.exec p_sql, array(maxtid, tid)
		end if
		'
		' Point all e-mail subscriptions to the new topic
		cor.forum.notif.moveNotify maxtid, tid
		'
		' Ensure count of topics/posts is correct in both - new and old forums
		p_sql = "UPDATE " & cor_dbpre & "forums SET stopics=stopics+1, sposts=sposts+" & arrMove(4,0) & " WHERE sid={0}"
		cor.dal.exec p_sql, array(tforumid)
		p_sql = "UPDATE " & cor_dbpre & "forums SET stopics=stopics-1, sposts=sposts-" & arrMove(4,0) & " WHERE sid={0}"
		cor.dal.exec p_sql, array(arrMove(6,0))

		'
		call updateLastPost(tforumid, maxtid)
		'
		' Write a log record to have some follow-up
		call cor.audit.writeLogDb(18, "The topic " & sTTitle & " [oldid:" & tid & ", newid:" & maxtid & "] was moved to forum " & tforumid)  
		'
		doMoveTopic = maxtid
	end function
	'
	private sub doEditTopic()
		sBodyn = cor.filter.BadWordFilter(sBodyn)
		sTTitle = cor.filter.BadWordFilter(sTTitle)
		
		p_sql = "UPDATE " & cor_dbpre & "topics SET ttitle = {0}, tbody = {1}, ttype = {2}, tdatechanged = {3}, tforumid = {4}, tmoved = 0 WHERE tid = {5}"
		cor.dal.exec p_sql, array(sTTitle, sBodyn, iTType, cor.butil.cvUxTime(Now()), fid, tid)
		'
		'also save updated data corresponding fictive post for the topic!!!
		p_sql  = "UPDATE " & cor_dbpre & "posts SET pbody = {0}, pdatechanged = {1} WHERE ptopicid = {2} AND pistopic = 1"
		cor.dal.exec p_sql, array(sBodyn, cor.butil.cvUxTime(Now()), tid)
		'
		' Set if member user has opted for e-mail notifications
		if  sNotify = "on" then
			cor.forum.notif.addnotify  fid, tid, cor_lUserID 
		else
			cor.forum.notif.delnotify  fid, tid, cor_lUserID 
		end if
	end sub
	'
	private sub doEditPost()
		sBodyn = cor.filter.BadWordFilter(sBodyn)
		p_sql = "UPDATE " & cor_dbpre & "posts SET pbody = {0}, pdatechanged = {1}, pforumid = {2}, ptopicid = {3} WHERE pid = {4}"
		cor.dal.exec p_sql, array(sBodyn, cor.butil.cvUxTime(Now()), fid, tid, pid)
	end sub
	'
	private sub doEditPoll()
		dim arrPoll, x_i,aPollAnswerCount
		aPollAnswerCount= cor.io.f("answersCountId")
		'
		' update topic/poll title
		cor.dal.exec "UPDATE " & cor_dbpre & "topics SET ttitle = {0}, tbody = {1}, ttype = {2} WHERE tid ={3}", array(sTTitle, sBodyn, iTType, tid)
		cor.dal.exec "UPDATE " & cor_dbpre & "polls SET poquestion = {0} WHERE poisquestion=1 AND potopicid ={1}", array(sTTitle, tid)
		'------------------0--------1----------
		p_sql = "SELECT po.poid, po.popollid FROM " & cor_dbpre & "polls AS po WHERE po.poisquestion<>1 AND po.potopicid = {0} ORDER BY po.poid"
		cor.dal.open arrPoll, p_sql, array(tid)
		'if cor.butil.eof(arrPoll) then exit sub
		'
		' update existing questions spelling
		for x_i = 0 to ubound(arrPoll,2)
			p_sql = "UPDATE " & cor_dbpre & "polls SET poanswer = {0} WHERE poid = {1}"
			cor.dal.exec p_sql, array(aPollAnswers(x_i), arrPoll(0,x_i))
		next
		'
		'if there were more questions added, add them to the poll 

		if ubound(arrPoll,2)+1 = ubound(aPollAnswers) Then '
			'(note arrPoll,2 = aPollAnswers actually would insert an additional entry)
			for x_i = ubound(arrPoll,2)+1 to ubound(aPollAnswers)
				cor.dal.exec "INSERT INTO " & cor_dbpre & "polls (poanswer,popollid,potopicid) VALUES ({0},{1},{2})", array(aPollAnswers(x_i), cor.io.f("poid"), tid)
			next
		end if
	end sub
	'
	private function doGoNextTopic()
		dim arrPN
		'
		cor.dal.openPaged arrPN, 1, 1, "SELECT t.tid, t.ttype, t.tdatelast FROM " & cor_dbpre & "topics AS t WHERE t.tforumid = {0} AND t.tid > {1} ORDER BY t.tid ASC", array(fid, tid)
		if  NOT cor.butil.eof(arrPN) Then
			doGoNextTopic = arrPN(0,0)
		else
			'cor.msg lngForaAlreadyLastTopic
			'
			'changed: show a message and stay on last topic
			cor.redirm lngForaAlreadyLastTopic, "fora_topic.asp?tid="& tid
			'
		end if
	end function
	'
	private function doGoPrevTopic()
		dim arrPN
		'
		cor.dal.openPaged arrPN, 1, 1, "SELECT t.tid, t.ttype, t.tdatelast FROM " & cor_dbpre & "topics AS t WHERE t.tforumid = {0} AND t.tid < {1} ORDER BY t.tid DESC", array(fid, tid)
		if  NOT cor.butil.eof(arrPN) Then
			doGoPrevTopic = arrPN(0,0)
		else
			'cor.msg lngForaAlreadyFirstTopic
			'
			'changed: show a message and stay on first topic
			cor.redirm lngForaAlreadyFirstTopic, "fora_topic.asp?tid="& tid
			'
		end if
	end function
	'
	private sub cor_cDoPollVote(voteid)
		dim arrPoll, poexpire, vpollid
		'
		' do not cast if poll has already expired
		poexpire = cor.dal.getValue("SELECT po.poexpires FROM " & cor_dbpre & "polls AS po WHERE po.potopicid ={0}", array(tid))
		if cor.butil.cvLcidTime(poexpire) < now() then exit sub
		'
		' Do not cast if member has already voted on this topic/poll
		p_sql = "SELECT pv.vid FROM " & cor_dbpre & "pollvotes AS pv WHERE pv.vmemberid ={0} AND pv.vtopicid = {1}"
		cor.dal.open arrPoll, p_sql, array(cor_lUserID, tid)
		if NOT cor.butil.eof(arrPoll) then exit sub
		'
		' cast vote for this user
		'
		' increase votes counter
		cor.dal.exec "UPDATE " & cor_dbpre & "polls SET povotes = povotes+1 WHERE poid ={0}", array(voteid)
		'
		' mark current user has already voted for this topic/poll, to avoid him voting again 
		vpollid = cor.dal.getValue("SELECT po.poid FROM " & cor_dbpre & "polls AS po WHERE po.poisquestion=1 AND po.potopicid ={0}", array(tid))
		p_sql = "INSERT INTO " & cor_dbpre & "pollvotes (vpollid,vmemberid,vtopicid,vanswerid) VALUES ({0},{1},{2},{3})"
		cor.dal.exec p_sql, array(vpollid, cor_lUserID, tid, voteid)
	end sub
	'
	'------------------------------------------------------------------------------------------------------------------------------------------
	'
	' Class Helpers
	'
	'------------------------------------------------------------------------------------------------------------------------------------------
	'
	' Get an IO array
	private function getaIO(ioType, ioName)
		dim aio  : redim aio(iPollAnswersMax)
		dim i, j : j = 0
		for i = 0 to ubound(aio)
			if lcase(ioType) = "f" Then
				if cor.io.f(ioName & i) <> "" Then aio(j) = cor.io.f(ioName & i) : j = j + 1
			else
				if cor.io.q(ioName & i) <> "" Then aio(j) = cor.io.q(ioName & i) : j = j + 1
			end if
		next
		redim preserve aio(j-1)
		getAIO = aio
	end function


	'
	' find topic/posts page to display
	private function findTopicPage()
		dim arrSearch, intFoundOnPage, x_i
		'
		findTopicPage = 1
		if pid=0 Then exit function
		p_sql = "SELECT p.pid FROM " & cor_dbpre & "posts AS p WHERE p.ptopicid = {0} AND p.pistopic = 0"
		cor.dal.open arrSearch, p_sql, array(tid)
		if cor.butil.eof(arrSearch) Then exit function
		for x_i = 0 to ubound(arrSearch,2)
			if  cor.filter.checkLng(arrSearch(0,x_i)) = cor.filter.checkLng(pid) Then
				intFoundOnPage = ( ubound(arrSearch,2) + 1) / postsPerPage
				if intFoundOnPage < 1 then
					intFoundOnPage = 1
				else
					if intFoundOnPage <> round(intFoundOnPage) then intFoundOnPage = round(intFoundOnPage) + 1
				end if
				exit for
			end if
		next
		findTopicPage = intFoundOnPage
	end function
	' Update last post made into a forum/topic
	private sub updateLastPost( intFid, intTid )
		dim llm
		call cor.forum.util.buildLatest(intFid, "both")
		if cLng(cor_lLastMid) < 1 then
			p_sql = "UPDATE " & cor_dbpre & "forums SET slast={0}, slasttid={1}, slastpid={2}, slasttime={3} WHERE sid={4}"
			cor.dal.exec p_sql, array(cor_sLastTopic, cor_lLastTid, cor_lLastPid, cor_dLastTime, intFid)
			p_sql = "UPDATE " & cor_dbpre & "topics SET tlast={0}, tlastpid={1}, tlasttime={2} WHERE tid={3}"
			cor.dal.exec p_sql, array(cor_sLastTopic, cor_lLastPid, cor_dLastTime, intTid)
		else
			p_sql = "UPDATE " & cor_dbpre & "forums SET slast={0}, slastmid={1}, slasttid={2}, slastpid={3}, slasttime={4} WHERE sid={5}"
			cor.dal.exec p_sql, array(cor_sLastTopic, llm, cor_lLastTid, cor_lLastPid, cor_dLastTime, intFid)
			p_sql = "UPDATE " & cor_dbpre & "topics SET tlast={0}, tlastmid={1}, tlastpid={2}, tlasttime={3} WHERE tid={4}"
			cor.dal.exec p_sql, array(cor_sLastTopic, cor_lLastMid, cor_lLastPid, cor_dLastTime, intTid)
		end if
	end sub
	'
	' Update number of topics/posts made by an user
	public function updatePostsByUser(byval user, byval intNumber)
	if isnumeric(user) and isnumeric(intNumber) then
		cor.dal.exec "UPDATE " & cor_dbpre & "members SET mnoofposts = mnoofposts + " & cor.filter.checkInt(intNumber) & _
		" WHERE MemberID = {0}", array(user)
	else 'update all
		if lcase(cor_sDatabaseType) = "access" then
			dim arrSearch
			p_sql= "SELECT pmemberid, COUNT(pid) FROM " & cor_dbpre & "posts GROUP BY pmemberid "
			cor.dal.open arrSearch, p_sql, array("")
			if cor.butil.eof(arrSearch) Then exit function
			for x_i = 0 to ubound(arrSearch,2)
				p_sql = "UPDATE " & cor_dbpre & "members SET mnoofposts = {0} WHERE memberid = {1}" 
				cor.dal.exec p_sql, array(arrSearch(1,x_i),arrSearch(0,x_i))
			next
		else	'selects don't work inside updates for Access...
			cor.dal.exec "UPDATE " & cor_dbpre & "members SET mnoofposts = (SELECT COUNT(pid) FROM " & cor_dbpre & "posts WHERE pmemberid = memberid)", array("")
		end if
	end if
		'cor.dal.exec "UPDATE " & cor_dbpre & "members SET mnoofposts = mnoofposts + " & cor.filter.checkInt(intNumber) & _
		'" WHERE MemberID = {0}", array(user)
	end function

	private sub DoPostForm()
		cor.str.wri "<script type=""text/javascript"">$(document).ready(function(){$('label[class^=errorfrm error]').remove();});</script>"
		cor.butil.checkNotVoid("[cor.forum:cor_cDoNewPostExt-POST],fid,tid")
		'
		if len(validateForum(bCaptcha))>0 then
			cor.str.wri validateForum(bCaptcha)
			call cor.butil.errorMsg(lngErrorDisplayLine1,lngValidationFail,True,3000,false,,0,false)
		else	' post form
			' Set from form checkbox if user wants email notification
			if sNotify = "on" Then
				cor.forum.notif.addnotify  fid, tid, cor_lUserID
			else
				cor.forum.notif.delnotify  fid, tid, cor_lUserID 
			end if
			pid_n = doNewPost()
			'ML make sure no page increment is needed so take out the pg from the qs so we force a find post page
			call cor.butil.successMsg(lngSuccessfullPost,lngSystemMessage,True,3000,true,"fora_topic.asp?tid=" & tid & "|" & pid_n & "#a" & pid_n,3500,false)
			'cor.str.wri "<script type=""text/javascript"">window.location='fora_topic.asp?tid=" & tid & "|" & pid_n & "#a" & pid_n & "';</script>"
		end if
		response.end
	end sub

	' test for captcha
	private function filterTextCaptcha()
		if cor_bPostback Then 
			if bCaptcha Then
				if NOT cor.butil.checkSecurityCode(sSecurityCode, sSessionName) Then
					filterTextCaptcha = false
					exit function
				end if
			end if
			filterTextCaptcha = true
		end If
	end function
	'

	public Function validateForum(bCaptcha)
		dim aPollAnswerCount, i, validateForumTest,valid_title
		aPollAnswerCount= cor.filter.checkInt(cor.io.f("answersCountId"))
		dim iDo : iDo = cor.filter.checkInt(cor.io.f("do"))
		
		if bCaptcha Then
			if not filterTextCaptcha() then cor.str.add "$('<label class=""errorfrm error"">" & lngReenterCaptcha & "</label>').insertAfter('#securityCode');"
		end if

		Select Case iDo
			Case cor_cDoNewPoll, cor_cDoEditPoll
				if len(sTTitle) <= 12 then valid_title = true else valid_title = false
			Case Else
				if len(sTTitle) <= 2 then valid_title = true else valid_title = false
		End Select

		dim valid_body : if len(sBodyn) <= 8 then valid_body = true else valid_body = false

		Select Case iDo
			Case cor_cDoNewTopic, cor_cDoEditTopic
				cor.butil.checkNotVoid("[cor.forum:cor_cDoNewTopic-POST],fid,sTTitle")
				' tests to return error messages
				if valid_title then cor.str.add "$('<label class=""errorfrm error"">" & lngForumErrTitle & "</label>').insertAfter('#ttitle');"
				if valid_body then cor.str.add "$('<label class=""errorfrm error"">" & lngTextIsEmpty & "</label>').insertBefore('#bodyn');"

			Case cor_cDoNewPost, cor_cDoEditPost, cor_cDoNewPostExt, cor_cDoNewPostExtQ
				' tests to return error messages
				if valid_body then cor.str.add "$('<label class=""errorfrm error"">" & lngTextIsEmpty & "</label>').insertBefore('#bodyn');"

			Case cor_cDoNewPoll, cor_cDoEditPoll
				' tests that return error messages
				if aPollAnswerCount=0 then  cor.str.add "$('<label class=""errorfrm error"">" & lngForumPollErrNumAnswer & "</label>').insertAfter('#addQuestion');"
				if valid_title then cor.str.add "$('<label class=""errorfrm error"">" & lngForumPollErrQuestion & "</label>').insertAfter('#ttitle');"
				for i = 0 to aPollAnswerCount
					if len(cor.io.f("answer"& i &""))<=1 then  cor.str.add "$('<label class=""errorfrm error"">" & lngForumPollErrAnswer & "</label>').insertAfter('#answer"& i &"');"
				next
				if valid_body then cor.str.add "$('<label class=""errorfrm error"">" & lngTextIsEmpty & "</label>').insertBefore('#bodyn');"

		End Select

		validateForumTest = cor.str.build
		If len(validateForumTest)=0 Then
			Select Case iDo
				Case cor_cDoNewTopic, cor_cDoEditTopic
					cor.butil.checkNotVoid("[cor.forum:cor_cDoNewTopic-POST],fid,sTTitle")
				Case cor_cDoNewPoll, cor_cDoEditPoll
					cor.butil.checkNotVoid("[cor.forum:cor_cDoNewPoll-POST],fid")
			End Select
		Else
			cor.str.wri "<script type=""text/javascript""><!-- <[CDATA["&vbcrlf&"$(document).ready(function(){$('label[class^=errorfrm error]').remove();});"&vbcrlf&"// ]]> --></script>"&vbcrlf
			cor.str.add "<script type=""text/javascript""><!-- <[CDATA["
			cor.str.add "try{"
			cor.str.add validateForumTest
			cor.str.add "	$('form#frmQuickReply button span').text('" & lngReply & "');"
			cor.str.add "	$('form#frmQuickReply button').attr('disabled', false);"
			cor.str.add "	$('form#frmTopic #tpcbtn span').text('" & lngReply & "');"
			cor.str.add "	$('form#frmTopic #tpcbtn').attr('disabled', false);"
			cor.str.add "	$('form#frmPoll #polbtn span').text('" & lngReply & "');"
			cor.str.add "	$('form#frmPoll #polbtn').attr('disabled', false);"
			cor.str.add "	$('form#frmPost #postbtn span').text('" & lngReply & "');"
			cor.str.add "	$('form#frmPost #postbtn').attr('disabled', false);"
			cor.str.add "	$('a.secureImg').click();"
			cor.str.add "} catch(e){}"
			cor.str.add "// ]]> --></script>"
			validateForum = cor.str.build
		End If
	End Function


end class
'
'<#late-binding hook
function boot_class_forum : boot_class_forum = true : end function
'/>

%>
